Package: scHOT
Type: Package
Title: single-cell higher order testing
Version: 1.25.0
Authors@R: c(
    person("Shila", "Ghazanfar", email = "shazanfar@gmail.com", role = c("aut", "cre")
    ),
    person("Yingxin", "Lin", email = "yingxin.lin@sydney.edu.au", role = c("aut")
    ))
Description: Single cell Higher Order Testing (scHOT) is an R package that facilitates testing changes in higher order structure of gene expression along either a developmental trajectory or across space. scHOT is general and modular in nature, can be run in multiple data contexts such as along a continuous trajectory, between discrete groups, and over spatial orientations; as well as accommodate any higher order measurement such as variability or correlation. scHOT meaningfully adds to first order effect testing, such as differential expression, and provides a framework for interrogating higher order interactions from single cell data.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0)
Imports: S4Vectors (>= 0.24.3), SingleCellExperiment, Matrix,
        SummarizedExperiment, IRanges, methods, stats, BiocParallel,
        reshape, ggplot2, igraph, grDevices, ggforce, graphics
RoxygenNote: 7.1.0
Suggests: knitr, markdown, rmarkdown, scater, scattermore, scales,
        matrixStats, deldir
VignetteBuilder: knitr
biocViews: GeneExpression, RNASeq, Sequencing, SingleCell, Software,
        Transcriptomics
git_url: https://git.bioconductor.org/packages/scHOT
git_branch: devel
git_last_commit: a5bc634
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:58:19 UTC; biocbuild
Author: Shila Ghazanfar [aut, cre],
  Yingxin Lin [aut]
Maintainer: Shila Ghazanfar <shazanfar@gmail.com>

# Generated by roxygen2: do not edit by hand

export("params<-")
export("positionColData<-")
export("positionType<-")
export("scHOT_output<-")
export("testingScaffold<-")
export("weightMatrix<-")
export(plotColouredExpression)
export(plotEgoNetwork)
export(plotHigherOrderSequence)
export(plotOrderedExpression)
export(scHOT)
export(scHOT_addTestingScaffold)
export(scHOT_buildFromMatrix)
export(scHOT_buildFromSCE)
export(scHOT_calculateGlobalHigherOrderFunction)
export(scHOT_calculateHigherOrderTestStatistics)
export(scHOT_estimatePvalues)
export(scHOT_performPermutationTest)
export(scHOT_plotPermutationDistributions)
export(scHOT_setPermutationScaffold)
export(scHOT_setWeightMatrix)
export(scHOT_stripOutput)
export(spatialWeightMatrix)
export(thin)
export(trajectoryWeightMatrix)
export(weightedPearson)
export(weightedSpearman)
export(weightedVariance)
export(weightedZIKendall)
export(weightedZISpearman)
exportClasses(scHOT)
import(ggplot2)
import(igraph)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(S4Vectors,DFrame)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(IRanges,NumericList)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,coolcat)
importFrom(S4Vectors,setValidity2)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(ggforce,geom_voronoi_tile)
importFrom(ggplot2,ggplot)
importFrom(graphics,plot)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,validObject)
importFrom(reshape,melt)
importFrom(reshape,sort_df)
importFrom(stats,dist)
importFrom(stats,loess)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,quantile)
importFrom(stats,sd)

# scHOT: single-cell higher order testing



## Installation


Install the following packages using `BiocManager`:

```r
# install.packages("BiocManager")
BiocManager::install(c("S4Vectors", "SummarizedExperiment", "SingleCellExperiment", 
"Matrix", "IRanges", "BiocParallel", "reshape", "ggplot2", "igraph", "grDevices", "ggforce"))
```

    
    
Then install `scHOT` using `devtools`:

```r
library(devtools)
devtools::install_github("shazanfar/scHOT")
```


## Vignette

You can find the vignette at this website: https://shazanfar.github.io/scHOT/.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(SingleCellExperiment)
library(ggplot2)
library(scHOT)
library(scater)
library(matrixStats)

## -----------------------------------------------------------------------------
data(liver)

liver_pseudotime_hep <- liver$liver_pseudotime_hep
liver_branch_hep <- liver$liver_branch_hep
first_branch_cells <- liver$first_branch_cells

## -----------------------------------------------------------------------------
gene_to_test <- as.matrix(c("Birc5", "H2afz", "Tacc3"))

## -----------------------------------------------------------------------------
scHOT_traj <- scHOT_buildFromMatrix(
  mat = liver_branch_hep[,first_branch_cells],
  cellData = list(pseudotime = liver_pseudotime_hep[first_branch_cells]),
  positionType = "trajectory",
  positionColData = "pseudotime")
scHOT_traj

## -----------------------------------------------------------------------------
scater::plotExpression(scHOT_traj, c("Sall4"),
                       exprs_values = "expression", x = "pseudotime")

## -----------------------------------------------------------------------------
scHOT_traj_wrap = scHOT(scHOT_traj,
                        testingScaffold = gene_to_test,
                        higherOrderFunction = matrixStats::weightedVar,
                        higherOrderFunctionType = "weighted",
                        numberPermutations = 50)

## -----------------------------------------------------------------------------
slot(scHOT_traj_wrap, "scHOT_output")
scHOT_traj_wrap@scHOT_output

slot(scHOT_traj_wrap, "scHOT_output")$higherOrderSequence

plotHigherOrderSequence(scHOT_traj_wrap, gene_to_test)

plotOrderedExpression(scHOT_traj_wrap, gene_to_test) + 
  facet_wrap(~gene, scales = "free_y")

## -----------------------------------------------------------------------------
scHOT_traj@testingScaffold
scHOT_traj <- scHOT_addTestingScaffold(scHOT_traj, gene_to_test)
scHOT_traj@testingScaffold

## -----------------------------------------------------------------------------
scHOT_traj@weightMatrix
scHOT_traj <- scHOT_setWeightMatrix(scHOT_traj,
                                    positionType = "trajectory",
                                    positionColData = c("pseudotime"),
                                    nrow.out = NULL,
                                    span = 0.25)
dim(scHOT_traj@weightMatrix)
class(scHOT_traj@weightMatrix)
plot(scHOT_traj@weightMatrix[50,])

## -----------------------------------------------------------------------------
scHOT_traj <- scHOT_setWeightMatrix(scHOT_traj,
                                    positionType = "trajectory",
                                    positionColData = c("pseudotime"),
                                    nrow.out = 50,
                                    span = 0.25)
dim(scHOT_traj@weightMatrix)
class(scHOT_traj@weightMatrix)
plot(scHOT_traj@weightMatrix[50,])

## -----------------------------------------------------------------------------
scHOT_traj <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_traj, 
  higherOrderFunction = matrixStats::weightedVar,
  higherOrderFunctionType = "weighted")

slot(scHOT_traj, "scHOT_output")
apply(assay(scHOT_traj, "expression")[c(gene_to_test),],1,var)

## -----------------------------------------------------------------------------
scHOT_traj <- scHOT_setPermutationScaffold(scHOT_traj, 
                                           numberPermutations = c(20,50,20),
                                           storePermutations = c(TRUE, FALSE, TRUE))
slot(scHOT_traj, "scHOT_output")

## -----------------------------------------------------------------------------
scHOT_traj <- scHOT_calculateHigherOrderTestStatistics(
  scHOT_traj,
  higherOrderSummaryFunction = sd)

slot(scHOT_traj, "scHOT_output")

## -----------------------------------------------------------------------------
system.time(scHOT_traj <- scHOT_performPermutationTest(
  scHOT_traj, 
  verbose = TRUE,
  parallel = FALSE))

slot(scHOT_traj, "scHOT_output")

## -----------------------------------------------------------------------------
scHOT_traj <- scHOT_estimatePvalues(scHOT_traj,
                                    nperm_estimate = 10000,
                                    maxDist = 5)
slot(scHOT_traj, "scHOT_output")

## -----------------------------------------------------------------------------
plotHigherOrderSequence(scHOT_traj, gene_to_test)

## -----------------------------------------------------------------------------
data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset

sce_MOB_subset

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                    assayName = "logcounts",
                                    positionType = "spatial",
                                    positionColData = c("x", "y"))

scHOT_spatial

## -----------------------------------------------------------------------------
pairs <- t(combn(rownames(sce_MOB_subset),2))
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
head(pairs)

set.seed(2020)
pairs <- pairs[sample(nrow(pairs), 20), ]
if (!"Arrb1_Mtor" %in% rownames(pairs)) {
pairs <- rbind(pairs, "Arrb1_Mtor" = c("Arrb1", "Mtor"))
}
if (!"Dnm1l_Fam63b" %in% rownames(pairs)) {
pairs <- rbind(pairs, "Dnm1l_Fam63b" = c("Dnm1l", "Fam63b"))
}

scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
scHOT_spatial@testingScaffold

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                       positionType = "spatial",
                                       nrow.out = NULL,
                                       span = 0.05)

dim(slot(scHOT_spatial, "weightMatrix"))

## -----------------------------------------------------------------------------
cellID = 75
ggplot(as.data.frame(colData(scHOT_spatial)), aes(x = -x, y = y)) + 
  geom_point(aes(colour = slot(scHOT_spatial, "weightMatrix")[cellID,],
                 size = slot(scHOT_spatial, "weightMatrix")[cellID,])) + 
  scale_colour_gradient(low = "black", high = "purple") + 
  scale_size_continuous(range = c(1,5)) +
  theme_classic() +
  guides(colour = guide_legend(title = "Spatial Weight"),
         size = guide_legend(title = "Spatial Weight")) + 
    ggtitle(paste0("Central cell: ", cellID))

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial, 
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")

slot(scHOT_spatial, "scHOT_output")

head(diag(cor(t(assay(scHOT_spatial, "expression")[pairs[,1],]),
         t(assay(scHOT_spatial, "expression")[pairs[,2],]),
         method = "spearman")))

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial, 
                                              numberPermutations = 50,
                                              numberScaffold = 10)

slot(scHOT_spatial, "scHOT_output")

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(scHOT_spatial)

slot(scHOT_spatial, "scHOT_output")

## -----------------------------------------------------------------------------
system.time(scHOT_spatial <- scHOT_performPermutationTest(
  scHOT_spatial, 
  verbose = TRUE,
  parallel = FALSE))

slot(scHOT_spatial, "scHOT_output")

## -----------------------------------------------------------------------------
scHOT_plotPermutationDistributions(scHOT_spatial)

scHOT_spatial <- scHOT_estimatePvalues(scHOT_spatial,
                                       nperm_estimate = 100,
                                       maxDist = 0.1)
slot(scHOT_spatial, "scHOT_output")

ggplot(as.data.frame(slot(scHOT_spatial, "scHOT_output")),
       aes(x = -log10(pvalPermutations), y = -log10(pvalEstimated))) + 
  geom_point() + 
  theme_classic() + 
  geom_abline(slope = 1, intercept = 0) +
  xlab("Permutation -log10(p-value)") +
  ylab("Estimated -log10(p-value)") +
  NULL

## -----------------------------------------------------------------------------
colData(scHOT_spatial)[, "-x"] <- -colData(scHOT_spatial)[, "x"]
plotHigherOrderSequence(scHOT_spatial, c("Dnm1l_Fam63b"),
                        positionColData = c("-x", "y"))

plotOrderedExpression(scHOT_spatial, c("Dnm1l", "Fam63b"),
                      positionColData = c("-x", "y"),
                      assayName = "expression")

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT_stripOutput(scHOT_spatial, force = TRUE)

scHOT_spatial
slot(scHOT_spatial, "scHOT_output")

## -----------------------------------------------------------------------------
scHOT_spatial <- scHOT(scHOT_spatial,
                       testingScaffold = pairs,
                       positionType = "spatial",
                       positionColData = c("x", "y"),
                       nrow.out = NULL,
                       higherOrderFunction = weightedSpearman,
                       higherOrderFunctionType = "weighted",
                       numberPermutations = 50,
                       numberScaffold = 10,
                       higherOrderSummaryFunction = sd,
                       parallel = FALSE,
                       verbose = FALSE,
                       span = 0.05)

slot(scHOT_spatial, "scHOT_output")

## -----------------------------------------------------------------------------
plotOrderedExpression(scHOT_spatial, c("Arrb1", "Mtor"),
                      positionColData = c("-x", "y"),
                      assayName = "expression")

plotHigherOrderSequence(scHOT_spatial, "Arrb1_Mtor",
                        positionColData = c("-x", "y"))

## -----------------------------------------------------------------------------
sessionInfo()

##############################################

#' the weightedZIKendall function calculates weighted Tau*,
#' where Tau* is described in Pimentel et al (2015)
#' doi:10.1016/j.spl.2014.09.002. This association measure
#' is defined for zero-inflated, non-negative random variables.
#'
#' @title weightedZIKendall
#' @param x x and y are non-negative data vectors
#' @param y x and y are non-negative data vectors
#' @param w weight vector, values should be between 0 and 1
#' @return \code{numeric} weighted Tau* association value between x and y

#' @examples
#'
#' x = pmax(0,rnorm(100))
#' y = pmax(0,rnorm(100))
#' w = runif(100)
#' weightedZIKendall(x,y,w)
#'
#' @export

weightedZIKendall <- function(x, y, w = 1) {

  if (any(x < 0 | y < 0)) stop("x and/or y values have negative values")

  if (length(x) != length(y)) stop("x and y should have the same length")

  if (length(w) == 1) w <- rep(w, length(x))

  posx = x > 0
  posy = y > 0

  pospos = posx & posy

  x_diff = outer(x[pospos],x[pospos], FUN = "-")
  y_diff = outer(y[pospos],y[pospos], FUN = "-")
  w_diff = outer(w[pospos],w[pospos])
  w_diff[lower.tri(w_diff, diag = TRUE)] <- 0

  p_conc = sum((x_diff*y_diff*upper.tri(x_diff) > 0)*w_diff)/sum(w_diff)
  p_disc = sum((x_diff*y_diff*upper.tri(x_diff) < 0)*w_diff)/sum(w_diff)

  tau_11 = p_conc - p_disc

  p_11 = sum(w*pospos)/sum(w)

  p_00 = sum(w*(!posx & !posy))/sum(w)

  p_01 = sum(w*(!posx & posy))/sum(w)

  p_10 = sum(w*(posx & !posy))/sum(w)

  # define p_1 and p_2
  # p_1 (weighted) proportion of times the x-values with y = 0 is greater than
  # the x-values with y > 0
  x_10 = x[!posy]
  x_11 = x[posy]

  w_x_10 = w[!posy]
  w_x_11 = w[posy]

  w_x_outer = outer(w_x_10, w_x_11)

  if (sum(w_x_outer) == 0) {
    p_1 <- 0
  } else {
    p_1 = sum(outer(x_10, x_11, FUN = ">")*w_x_outer) / sum(w_x_outer)
  }

  # analogous for y
  y_10 = y[!posx]
  y_11 = y[posx]

  w_y_10 = w[!posx]
  w_y_11 = w[posx]

  w_y_outer = outer(w_y_10, w_y_11)

  if (sum(w_y_outer) == 0) {
    p_2 <- 0
  } else {
    p_2 = sum(outer(y_10, y_11, FUN = ">")*w_y_outer) / sum(w_y_outer)
  }

  tauStar = (p_11^2)*tau_11 +
    2*(p_00*p_11 - p_01*p_10) +
    2*p_11*(p_10*(1 - 2*p_1) + p_01*(1 - 2*p_2))

  return(tauStar)
}



##############################################

#' the weightedPearson function
#'
#' @title weightedPearson
#' @param w weight vector, values should be between 0 and 1
#' @param x x and y are data vectors
#' @param y x and y are data vectors
#' @return \code{numeric} weighted correlation value between x and y

#' @examples
#'
#' x = rnorm(100)
#' y = rnorm(100)
#' w = runif(100)
#' weightedPearson(x,y,w)
#'
#' @export

weightedPearson = function(x, y, w = 1) {

  if (length(x) != length(y)) stop("data must be the same length")

  if (length(w) == 1) {
    w <- rep(w, length(x))
  }

  nw = sum(w)
  wssx = nw * sum(w * (x^2)) - sum(w * x)^2
  wssy = nw * sum(w * (y^2)) - sum(w * y)^2
  wssxy = nw * sum(w * x * y) - sum(w * x) * sum(w * y)
  wcor = wssxy/sqrt(wssx * wssy)
  return(wcor)
}



##############################################

#' the weightedSpearman function
#'
#' @title weightedSpearman
#' @param w weight vector, values should be between 0 and 1
#' @param x x and y are data vectors
#' @param y x and y are data vectors
#' @return \code{numeric} weighted correlation value between x and y

#' @examples
#'
#' x = rnorm(100)
#' y = rnorm(100)
#' w = runif(100)
#' weightedSpearman(x,y,w)
#'
#' @export

weightedSpearman = function(x, y, w = 1) {

  if (length(x) != length(y)) {
    stop("x and y should have the same length")
  }
  if (length(w) == 1) {
    w <- rep(w, length(x))
  }

  keep = w > 0

  xr = rank(x[keep])
  yr = rank(y[keep])
  return(weightedPearson(x = xr, y = yr, w = w[keep]))
}






##############################################

#' the weightedVariance function
#'
#' @title weightedVariance
#' @param w weight vector, values should be between 0 and 1
#' @param x x is a data vector
#' @param y default to NULL, if given it is ignored
#' @return \code{numeric} weighted variance value for x

#' @examples
#'
#' x = rnorm(100)
#' w = runif(100)
#' weightedVariance(x,w = w)
#'
#' @export

weightedVariance = function(x, y = NULL, w) {
  # args x,w (if y given, it is ignored)

  w <- w/sum(w)
  nw <- sum(w)
  n <- sum(w != 0)
  wssx <- nw * sum(w * (x^2)) - sum(w * x)^2
  wvar <- (n/(n - 1))*wssx
  return(wvar)

}




##############################################

#' the weightedZISpearman function calculates weighted rho\*,
#' where rho\* is described in Pimentel et al (2009).
#' This association measure is defined for zero-inflated,
#' non-negative random variables.
#'
#' @title weightedZISpearman
#' @param w weight vector, values should be between 0 and 1
#' @param x x and y are non-negative data vectors
#' @param y x and y are non-negative data vectors
#' @return \code{numeric} weighted rho* association value between x and y
#'
#'  Pimentel, Ronald Silva, "Kendall's Tau and Spearman's Rho for
#'  Zero-Inflated Data" (2009). Dissertations. 721.
#'  https://scholarworks.wmich.edu/dissertations/721

#' @examples
#'
#' x = pmax(0,rnorm(100))
#' y = pmax(0,rnorm(100))
#' w = runif(100)
#' weightedZISpearman(x,y,w)
#'
#' @export

weightedZISpearman <- function(x, y, w = 1) {

  # needs the original values, not the ranks

  if (any(x < 0 | y < 0)) {
    stop("x and/or y values have negative values")
  }
  if (length(x) != length(y)) {
    stop("x and y should have the same length")
  }
  if (length(w) == 1) {
    w <- rep(w, length(x))
  }

  posx = x > 0
  posy = y > 0
  pospos = posx & posy

  p_11 = sum(w * pospos)/sum(w)
  p_00 = sum(w * (!posx & !posy))/sum(w)
  p_01 = sum(w * (!posx & posy))/sum(w)
  p_10 = sum(w * (posx & !posy))/sum(w)
  rho_11 = weightedSpearman(x = x[pospos], y = y[pospos], w = w[pospos])
  rho_star = p_11 * (p_01 + p_11) * (p_10 + p_11) * rho_11 +
    3*(p_00 * p_11 - p_10 * p_01)

  if (is.na(rho_star)) {
    print("Zero inflated Spearman correlation is undefined,
          returning Spearman correlation")
    rho = weightedSpearman(x = x, y = y, w = w)
    return(rho)
  }

  return(rho_star)
}

#' Create scHOT object from a matrix
#'
#' @title scHOT_buildFromMatrix
#'
#' @param mat A matrix with rows for genes and columns for cells
#' @param cellData A dataframe or DataFrame object with rows for cells
#' @param positionType A string indicating the position type,
#' either "trajectory" or "spatial"
#' @param positionColData Strings indicate the position information
#' stored in colData. If positionType is "trajectory" then
#' positionColData should be a sortable vector
#' if positionType is "spatial" then positionColData should
#' be a matrix type object.
#'
#' @return A scHOT object
#'
#' @importFrom S4Vectors DataFrame SimpleList
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom methods as validObject is
#'
#' @examples
#'
#' dat <- rbind(rnorm(50), rnorm(50), rnorm(50))
#' colnames(dat) <- paste0("cell_", 1:ncol(dat))
#' rownames(dat) <- c("gene_1","gene_2", "gene_2")
#'
#' scHOT <- scHOT_buildFromMatrix(dat, cellData = data.frame(1:ncol(dat)))
#'
#' @export



scHOT_buildFromMatrix <- function(mat, cellData = NULL, positionType = NULL,
                                  positionColData = NULL) {
  if (is.null(cellData)) {
    cellData <- S4Vectors::DataFrame(row.names = colnames(mat))
  }

  if (!is(cellData, "DataFrame")) {
    cellData <- S4Vectors::DataFrame(cellData)
  }

  SCE <- SingleCellExperiment(assays = S4Vectors::SimpleList(expression = mat),
                              colData = cellData)

  # scHOT <- methods::as(SCE, "scHOT")

  # scHOT@positionType <- positionType

  # scHOT@positionColData <- positionColData

  scHOT <- .scHOT(SCE,
                  positionType = positionType,
                  positionColData = positionColData)

  methods::validObject(scHOT)

  return(scHOT)
}

###################################################################



#' Create scHOT object from a SingleCellExperiment object
#'
#' @title scHOT_buildFromSCE
#'
#' @param sce A SingleCellExperiment object
#' @param assayName is a single assay to pull out from sce as
#' the expression matrix input of scHOT
#' @param positionType A string indicates the position type,
#' either trajectory or spatial
#' @param positionColData Strings indicate the position information
#' stored in colData.
#' If positionType is "trajectory" then positionColData
#' should be a sortable vector
#' if positionType is "spatial" then positionColData
#' should be a matrix type object.
#'
#'
#' @return A scHOT object
#'
#'
#' @importFrom S4Vectors SimpleList
#' @importFrom SingleCellExperiment SingleCellExperiment
#' @importFrom SummarizedExperiment colData assay
#' @importFrom methods as validObject
#' @examples
#'
#' library(SingleCellExperiment)
#' dat <- rbind(rnorm(50), rnorm(50), rnorm(50))
#' colnames(dat) <- paste0("cell_", 1:ncol(dat))
#' rownames(dat) <- c("gene_1","gene_2", "gene_2")
#'
#' sce <- SingleCellExperiment::SingleCellExperiment(assays =
#' S4Vectors::SimpleList(counts = dat))
#' scHOT <- scHOT_buildFromSCE(sce)
#'
#' @export



scHOT_buildFromSCE <- function(sce,
                               assayName = "counts",
                               positionType = NULL,
                               positionColData = NULL) {

  # will keep the colData but nothing else

  SCE <- SingleCellExperiment(assays = SimpleList(expression =
                                                    assay(sce, assayName)),
                              colData = colData(sce))


  # scHOT <- methods::as(SCE, "scHOT")

  # scHOT@positionType <- positionType

  # scHOT@positionColData <- positionColData

  scHOT <- .scHOT(SCE,
                  positionType = positionType,
                  positionColData = positionColData)

  methods::validObject(scHOT)

  return(scHOT)
}



#' Add a testing scaffold to a scHOT object
#'
#' @title scHOT_addTestingScaffold
#'
#' @param scHOT A scHOT object
#' @param testingScaffold A matrix with rows for each testing combination,
#' and columns for level of dimensionality (1 for single gene etc.)
#'
#' @return A scHOT object with slot testingScaffold saved
#'
#' @examples
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#' scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#'
#'
#' @export


scHOT_addTestingScaffold <- function(scHOT, testingScaffold) {
  # scaffold is a matrix with rows for each testing combination
  # and columns for level of dimensionality (1 for single gene etc.)

  # check if any are not found in data
  genesFound <- rowSums(apply(testingScaffold, seq_len(2),
                              function(x) x %in% rownames(scHOT)))

  if (any(genesFound != ncol(testingScaffold))) {
    warning("Some genes not found in data, removing these..")
  }

  # Remove the genes that are not in data
  testingScaffold <- testingScaffold[genesFound == ncol(testingScaffold), , drop = FALSE]

  colnames(testingScaffold) <- paste0("gene_", seq_len(ncol(testingScaffold)))

  # scHOT@testingScaffold <- testingScaffold
  testingScaffold(scHOT) <- testingScaffold

  return(scHOT)
}


###################################################################


#' Create weight matrix for trajectory data
#'
#' @title trajectoryWeightMatrix
#'
#' @param n indicates the number of cels
#' @param type Type of weight matrix, one of "triangular" (default),
#' "block", and "harmonic"
#' @param span proportion of samples to include on either side, default is 0.25
#'
#' @return A weighted matrix
#'
#' @examples
#'
#' W <- trajectoryWeightMatrix(100)
#' W <- trajectoryWeightMatrix(100, type = "triangular")
#' W <- trajectoryWeightMatrix(100, type = "block")
#' W <- trajectoryWeightMatrix(100, type = "harmonic")
#'
#' @export



trajectoryWeightMatrix <- function(n, type = NULL, span = NULL) {
  # n indicates the number of cels

  if (is.null(type)) {
    type = "triangular"
    message("type not specified, defaulting to triangular")
  }

  if (!type %in% (c("harmonic", "triangular", "block"))) {
    type = "triangular"
    message("type given is not one of \"harmonic\",\"triangular\",
            or \"block\", defaulting to \"triangular\"")
  }

  if (type != "harmonic") {
    if (is.null(span)) {
      span = 0.25
      message("span not specified, defaulting to 0.25")
    }
    if (span < 0 | span > 1) {
      span = 0.25
      message("span specified not between 0 and 1, defaulting to 0.25")
    }
  }

  W = matrix(0, nrow = n, ncol = n)
  spanSamples = ceiling(span * n)

  if (type == "harmonic") {
    for (i in seq_len(n)) {
      j = seq_len(n)
      W[i, j] <- 1/(1 + abs(i - j))
    }
  }

  if (type == "triangular") {
    for (i in seq_len(n)) {
      j = seq_len(n)
      W[i, j] <- (abs(j - i) <= spanSamples) *
        ((spanSamples + 1) - abs(j - i))/(spanSamples + 1)
    }
  }

  if (type == "block") {
    for (i in seq_len(n)) {
      j = seq_len(n)
      W[i, j] <- (abs(j - i) <= spanSamples) * 1
    }
  }

  return(W)
}

###################################################################


#' Create weight matrix for spatial data
#'
#' @title spatialWeightMatrix
#'
#' @param x a matrix with rows corresponding to cells and columns
#' corresponding to dimensions to calculate Euclidean distance
#' @param span proportion of samples to include on either side,
#' default is 13/(number of rows in `x`), corresponding roughly
#' to points within a diamond shape distance away
#'
#' @return A weighted matrix
#'
#' @examples
#'
#' spat_x <- rnorm(50)
#' spat_y <- rnorm(50)
#' spat_coord <- cbind(spat_x, spat_y)
#' W <- spatialWeightMatrix(spat_coord)
#'
#' @importFrom stats dist
#'
#' @export

spatialWeightMatrix <- function(x, span = NULL) {

  # x is a matrix with rows corresponding to cells and columns
  # corresponding to dimensions to calculate distance

  n = nrow(x)

  if (is.null(span)) {
    span = 13/n
    if (span > 1) span = 0.5
    message("span not specified, defaulting to ", round(span, 2))
  }

  # calculate euclidean distance of points on 2D
  # coords = cbind(x,y)
  coords = as.matrix(x)
  d = as.matrix(stats::dist(coords))

  # extract a weights vector per cell

  W_raw = sapply(seq_len(n), function(cell) {
    dvec = d[cell,]
    vals = rep(0, n)
    #vals[order(dvec)[1:ceiling(span*n)]] = seq(1, 0, length.out = ceiling(span*n))
    vals[order(dvec)[seq_len(ceiling(span*n))]] =
      seq(1, 0, length.out = ceiling(span*n))
    return(vals)
  }, simplify = FALSE)

  W = do.call(rbind, W_raw)

  return(W)

}

###################################################################


#' The thin function extracts the rows of a matrix evenly so that
#' roughly n number of rows remain. Used for thinning down
#' the weight matrix to speed up overall computation.
#'
#' @title thin
#' @param W matrix
#' @param n rough number of rows to keep
#'
#' @return \code{matrix} of thinned matrix keeping only roughly n rows.
#'
#' @examples
#'
#' W = trajectoryWeightMatrix(500)
#' W_small = thin(W, n = 100)
#'
#' @export

thin = function(W, n = 100) {
  # given a matrix W, extract the rows so that
  # the total number of rows is roughly n

  N = nrow(W)

  if (n == N) return(W)

  if (n > N) stop("need to set n to less than nrow(W)")

  new_n = floor(N/n)

  index = seq(from = 1, to = N, by = new_n)

  W <- W[index,]
  rownames(W) <- index
  return(W)
}


###################################################################


#' Create scHOT object from a SingleCellExperiment object
#'
#' @title scHOT_setWeightMatrix
#'
#' @param scHOT A scHOT object
#' @param weightMatrix A matrix indicating the weight matrix for
#' scHOT analysis, such as the output from `trajectoryWeightMatrix`
#' or `spatialWeightMatrix`. If this is not NULL then other parameters
#' are ignored.
#' @param positionType A string indicating the position type,
#' either "trajectory" or "spatial"
#' @param positionColData Either trajectory or spatial information
#' for each sample. If positionType is "trajectory"
#' then positionColData should be a character or
#' numeric indicating the subset of colData of the scHOT object.
#' If positionType is "spatial" then positionColData
#' should be a character or numeric vector indicating
#' the subset of colData that give the full spatial coordinates.
#' @param nrow.out The number of weightings to include for testing,
#' a smaller value is faster for computation
#' @param averageAcrossTrajectoryTies Logical indicating whether ties
#' in the trajectory should be given the same local weights
#' @param ... parameters for function trajectoryWeightMatrix
#'  or spatialWeightMatrix
#'
#' @return A scHOT object with slot weightMatrix saved
#'
#' @examples
#'
#'  data(MOB_subset)
#'  sce_MOB_subset <- MOB_subset$sce_MOB_subset
#'  scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#'  pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#'  rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#'  scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#'  scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
#'                                        positionColData = c("x","y"),
#'                                         positionType = "spatial",
#'                                         nrow.out = NULL,
#'                                         span = 0.05)
#'
#' @importFrom methods as is
#'
#' @export

scHOT_setWeightMatrix <- function(scHOT,
                                  weightMatrix = NULL,
                                  positionType = NULL,
                                  positionColData = NULL,
                                  nrow.out = NULL,
                                  averageAcrossTrajectoryTies = FALSE,
                                  ...) {

  # either set own weight matrix in object or generate a new one using parameters
  # if positionType is "trajectory" then positionColData should be a sortable vector
  # if positionType is "spatial" then positionColData should be a matrix type object


  # input check

  if (!is(scHOT, "scHOT")) {
    stop("scHOT needs to be scHOT class")
  }

  if (is.null(positionType)) {

    if (is.null(scHOT@positionType)) {
      stop("Both positionType and scHOT@positionType are NULL.")
    } else {
      positionType <- scHOT@positionType
    }

  }

  positionType <- match.arg(positionType, c("trajectory","spatial"), several.ok = FALSE)



  if (is.null(positionColData)) {

    if (is.null(scHOT@positionColData)) {
      stop("Both positionColData and scHOT@positionColData are NULL.")
    } else {
      positionColData <- scHOT@positionColData
    }

  }

  if (!all(positionColData %in% colnames(colData(scHOT)))) {
    stop("at least one positionColData column names not found in colData(scHOT)")
  }


  if (!is.null(weightMatrix)) {

    if (ncol(weightMatrix) != ncol(scHOT)) {
      stop("Provided weightMatrix must have same number of columns as scHOT object!")
    }

    rownames(weightMatrix) <- seq_len(nrow(weightMatrix))

  } else {


    message("weightMatrix not provided, generating one using parameter settings...")


    n = ncol(scHOT)

    # if (!positionType %in% c("trajectory","spatial")) {
    #   stop("select either \"trajectory\", or \"spatial\" as positionType")
    # }


    if (positionType == "trajectory") {

      if (length(positionColData) > 1) {
        stop("only one positionColData column name to be given for trajectory")
      }

      weightMatrix = trajectoryWeightMatrix(n, ...)
      colnames(weightMatrix) <- colnames(scHOT)
      weightMatrix = weightMatrix[ , order(colData(scHOT)[, positionColData],
                                           na.last = NA), drop = FALSE]

      if (averageAcrossTrajectoryTies) {
        # average across ties
        weightMatrixTied = t(apply(weightMatrix,
                                   1, function(x,
                                               y = colData(scHOT)[, positionColData]){
                                     unsplit(tapply(x,y,mean),y)
                                   }))
        colnames(weightMatrixTied) <- colnames(weightMatrix)
        weightMatrix = weightMatrixTied
      }
    }

    if (positionType == "spatial") {
      weightMatrix = spatialWeightMatrix(as.matrix(colData(scHOT)[, positionColData]), ...)
      colnames(weightMatrix) <- colnames(scHOT)
      # rownames(weightMatrix) <- colnames(scHOT)
    }

    if (!is.null(nrow.out)) {
      weightMatrix <- thin(weightMatrix, n = nrow.out)
    } else {
      rownames(weightMatrix) <- seq_len(nrow(weightMatrix))
    }

  }

  weightMatrix <- methods::as(weightMatrix, "dgCMatrix")

  # scHOT@weightMatrix <- weightMatrix
  weightMatrix(scHOT) <- weightMatrix


  # scHOT@positionType <- positionType
  positionType(scHOT) <- positionType
  # scHOT@positionColData <- positionColData
  positionColData(scHOT) <- positionColData


  return(scHOT)
}

###################################################################


functionOverScaffoldWithWeights = function(testingScaffold,
                                           expressionData,
                                           higherOrderFunction,
                                           weightMatrix) {

  # functionOverScaffoldWithWeights takes in a set of weights and
  # applies a function to it selectively in such a way that it's
  # equivalent to a block weight matrix set up
  # higherOrderFunction must have same number of arguments as testingScaffold

  sapply(seq_len(nrow(weightMatrix)), function(i) {
    apply(testingScaffold, 1, function(x) {
      argvals = split(expressionData[x,
                                     weightMatrix[i, ,
                                                  drop = FALSE] > 0,
                                     drop = FALSE],
                      seq_len(length(x)))
      #names(argvals) <- names(formals(higherOrderFunction))[1:length(argvals)]
      names(argvals) <- names(formals(higherOrderFunction))[seq_len(length(argvals))]
      return(do.call(higherOrderFunction, args = argvals))
    })
  })
}

###################################################################

weightedFunctionOverScaffold = function(testingScaffold,
                                        expressionData,
                                        weightedHigherOrderFunction,
                                        weightMatrix) {

  # weightedFunctionOverScaffold takes in a set of weights
  # and applies a weighted function to it
  #  weightedFunction must have sample weighting as its last argument

  sapply(seq_len(nrow(weightMatrix)), function(i) {
    apply(testingScaffold, 1, function(x) {
      # argvals = split(rbind(expressionData[x, , drop = FALSE],
      #                       w = weightMatrix[i, , drop = FALSE]),
      #                 1:(length(x) + 1))
      argvals = split(rbind(expressionData[x, , drop = FALSE],
                            w = weightMatrix[i, , drop = FALSE]),
                      seq_len((length(x) + 1)))
      #names(argvals) <- names(formals(weightedHigherOrderFunction))[1:length(argvals)]
      names(argvals) <- names(formals(weightedHigherOrderFunction))[seq_len(length(argvals))]
      return(do.call(weightedHigherOrderFunction, args = argvals))
    })
  })
}


###################################################################


#' Calculates the global higher order function
#'
#' @title scHOT_calculateGlobalHigherOrderFunction
#'
#' @description this calculates the global higher order function
#' and stores it in the output
#' if these aren't found in the params slot
#' then they need to be specified here
#'
#' @param scHOT A scHOT object
#' @param higherOrderFunction A function object indicating the higher order function
#' @param higherOrderFunctionType is "weighted" or "unweighted", determines if there
#' is a weighting argument in the higher order function
#'
#' @return A scHOT object with scHOT_output$globalHigherOrderFunction in
#' slot scHOT_output saved
#' @examples
#'
#'  data(MOB_subset)
#'  sce_MOB_subset <- MOB_subset$sce_MOB_subset
#'  scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#' scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#'  scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
#'                                        positionColData = c("x","y"),
#'                                         positionType = "spatial",
#'                                         nrow.out = NULL,
#'                                         span = 0.05)
#' scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
#'   scHOT_spatial,
#'   higherOrderFunction = weightedSpearman,
#'   higherOrderFunctionType = "weighted")
#'
#' @importFrom SummarizedExperiment assay
#' @importFrom methods is
#'
#'
#' @export




scHOT_calculateGlobalHigherOrderFunction <- function(
  scHOT,
  higherOrderFunction = NULL,
  higherOrderFunctionType = NULL) {

  # this calculates the global higher order function and stores it in the output
  # if these aren't found in the params slot then they need to be specified here
  # higherOrderFunctionType is weighted or unweighted, determines if there
  # is a weighting argument in the higher order function

  if (!is.null(higherOrderFunctionType)) {
    message("higherOrderFunctionType given will replace any stored param")
    scHOT@params$higherOrderFunctionType <- higherOrderFunctionType
  }

  if (!higherOrderFunctionType %in% c("weighted","unweighted")) {
    stop("higherOrderFunctionType must be either \"weighted\"
         or \"unweighted\"")
  }

  if (is.null(higherOrderFunctionType)) {
    if (is.null(scHOT@params$higherOrderFunctionType)) {
      stop("higherOrderFunctionType not given and not saved in params,
           please provide as either \"weighted\" or \"unweighted\"")
    } else {
      higherOrderFunctionType <- scHOT@params$higherOrderFunctionType
    }
  }

  if (!is.null(higherOrderFunction)) {
    message("higherOrderFunction given will replace any stored param")
    scHOT@params$higherOrderFunction <- higherOrderFunction
  }
  if (!is(higherOrderFunction, "function")) {
    stop("higherOrderFunction must be a function object")
  }
  if (is.null(higherOrderFunction)) {
    if (is.null(scHOT@params$higherOrderFunction)) {
      stop("higherOrderFunction not given and not saved in params,
           please provide as function object")
    } else {
      higherOrderFunction <- scHOT@params$higherOrderFunction
    }
  }

  testingScaffold = scHOT@testingScaffold

  if (is.null(testingScaffold)) {
    stop("No testingScaffold found in the scHOT object,
         please provide one!")
  }

  expressionData = SummarizedExperiment::assay(scHOT, "expression")

  if (is.null(expressionData)) {
    stop("No expressionData found, please provide an assay(scHOT,
         \"expression\") slot")
  }

  weightMatrix_trivial = matrix(1,nrow = 1, ncol = ncol(expressionData))

  if (higherOrderFunctionType == "unweighted") {
    global = functionOverScaffoldWithWeights(testingScaffold,
                                             expressionData,
                                             higherOrderFunction,
                                             weightMatrix_trivial)
  } else {
    # for weighted
    global = weightedFunctionOverScaffold(testingScaffold,
                                          expressionData,
                                          higherOrderFunction,
                                          weightMatrix_trivial)
  }

  if (nrow(scHOT@scHOT_output) == 0) {
    # scHOT@scHOT_output = DataFrame(testingScaffold)
    scHOT_output(scHOT) <- DataFrame(testingScaffold)
  }

  scHOT <- scHOT_stripOutput(scHOT, force = FALSE)

  scHOT@scHOT_output$globalHigherOrderFunction <- global

  return(scHOT)
}

###################################################################

stratifiedSample = function(stats, length = 100) {
  nsamps = 5
  ranges = range(stats[is.finite(stats)])
  fac = cut(stats, seq(from = ranges[1] - 1e-04, to = ranges[2],
                       length.out = ceiling(length/nsamps)))
  # sampleindices = unlist(tapply(1:length(stats), fac, function(x) {
  #   if (length(x) < nsamps)
  #     return(x)
  #   sample(x, nsamps, replace = FALSE)
  # }))
  sampleindices = unlist(tapply(seq_len(length(stats)), fac, function(x) {
    if (length(x) < nsamps)
      return(x)
    sample(x, nsamps, replace = FALSE)
  }))
  sampleindices = unique(sampleindices[!is.na(sampleindices)])
  return(sampleindices)
}




###################################################################


#' Set permutation scaffold
#'
#' @title scHOT_setPermutationScaffold
#'
#' @param scHOT A scHOT object
#' @param numberPermutations The number of permutations, set as 1000 by default
#' @param numberScaffold The number of scaffold, set as 100 by default, minimum 6.
#' if you want all combinations to do permutations then set,
#' numberScaffold much higher than the testingScaffold
#' @param storePermutations  a logical flag on whether
#' Permutations should be stored, or discarded once used
#'
#' @return A scHOT object with storePermutations in slot scHOT_output saved
#'
#' @examples
#'  data(MOB_subset)
#'  sce_MOB_subset <- MOB_subset$sce_MOB_subset
#'  scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#' scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#' scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
#'                                        positionColData = c("x","y"),
#'                                         positionType = "spatial",
#'                                         nrow.out = NULL,
#'                                         span = 0.05)
#' scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
#'   scHOT_spatial,
#'   higherOrderFunction = weightedSpearman,
#'   higherOrderFunctionType = "weighted")
#' scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
#'                                               numberPermutations = 100)
#'
#' @export




scHOT_setPermutationScaffold = function(scHOT,
                                        numberPermutations = 1000,
                                        numberScaffold = 100,
                                        storePermutations = TRUE) {

  # if you want all combinations to do permutations then set
  # numberScaffold much higher than the testingScaffold
  # storePermutations is a logical flag on whether
  # Permutations should be stored,
  # or discarded

  testingScaffold = scHOT@testingScaffold

  if (is.null(testingScaffold)) {
    stop("No testingScaffold found in the scHOT object, please provide one!")
  }

  if (nrow(scHOT@scHOT_output) == 0) {
    # scHOT@scHOT_output = DataFrame(testingScaffold)
    scHOT_output(scHOT) <- DataFrame(testingScaffold)
  }

  scHOT <- scHOT_stripOutput(scHOT, force = FALSE)

  if (numberScaffold > nrow(scHOT@testingScaffold)) {
    message("numberScaffold set higher than the scaffold, setting permutation number for all tests")
    scHOT@scHOT_output$numberPermutations = numberPermutations
  } else {

    if (numberScaffold < 6) {
      message("numberScaffold set lower than 6, resetting to 6")
      numberScaffold <- 6
    }

    if (is.null(scHOT@scHOT_output$globalHigherOrderFunction)) {
      stop("need scHOT@scHOT_output$globalHigherOrderFunction to take random stratified sample")
    }

    scHOT@scHOT_output$numberPermutations = 0
    scHOT@scHOT_output$numberPermutations[
      stratifiedSample(
        scHOT@scHOT_output$globalHigherOrderFunction,
        length = numberScaffold)] <- numberPermutations
  }
  scHOT@scHOT_output$storePermutations = storePermutations

  return(scHOT)
}


###################################################################


#' Strip the scHOT output
#'
#' @title scHOT_stripOutput
#'
#' @param scHOT A scHOT object
#' @param force A logical indicates whther forcing stripping the scHOT output
#' @param store A logical flag on whether the scHOT should be stored as .rds file
#' @param file_name A string indicates the file name of the scHOT will be stored
#' @examples
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#'
#' scHOT_spatial <- scHOT_stripOutput(scHOT_spatial)
#'
#' @return A scHOT object with scHOT_output striped
#'
#' @examples
#'
#'
#' @export



scHOT_stripOutput <- function(scHOT, force = TRUE,
                              store = FALSE,
                              file_name = NULL) {

  scHOT_output <- scHOT@scHOT_output

  if (!is.null(scHOT_output)) {
    # current_testing <- paste(scHOT@testingScaffold[, 1],
    # scHOT@testingScaffold[, 2], sep = "_")

    if (nrow(scHOT@testingScaffold) == 1) {
      current_testing <- apply(scHOT@testingScaffold, 1, paste, collapse = "_")
      current_output_testing <- apply(scHOT_output[, grepl("gene_[0-9]",
                                                           colnames(scHOT_output)),
                                                   drop = FALSE], 1, paste,
                                      collapse = "_")
    } else {
      current_testing <- apply(scHOT@testingScaffold, 1, paste, sep = "_")
      current_output_testing <- apply(scHOT_output[, grepl("gene_[0-9]",
                                                           colnames(scHOT_output)),
                                                   drop = FALSE], 1, paste,
                                      sep = "_")
    }

    # current_testing <- apply(scHOT@testingScaffold, 1, paste, sep = "_")
    # current_output_testing <- paste(scHOT_output$gene_1,
    # scHOT_output$gene_2, sep = "_")
    # current_output_testing <- apply(scHOT_output[, grepl("gene_[0-9]",
    # colnames(scHOT_output)), drop = FALSE], 1, paste, sep = "_")

    if (force | !all(current_testing %in% current_output_testing) | length(current_testing) != length(current_output_testing)) {

      warnings("The current testing scaffold does not match with the
               current scHOT_output. The scHOT_output will be stripped out.")
      scHOT_output <- DataFrame(NULL)

      if (store) {

        if (is.null(file_name)) {
          file_name <- "scHOT.Rds"
        } else {
          file_name <- paste0(file_name, ".Rds")
        }

        message("The current scHOT wil be stored as ", file_name)
        saveRDS(scHOT, file = file_name)

      }

    }

  } else {
    scHOT_output <- DataFrame(NULL)
  }



  # scHOT@scHOT_output <- scHOT_output
  scHOT_output(scHOT) <- scHOT_output

  return(scHOT)
}



###################################################################
# Accessor functions for scHOT objects

#' Setter functions for scHOT objects
#'
#' @title Setter functions for scHOT objects
#' @param x A scHOT object
#' @param value The value the slot should take
#'
#' @examples
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#'
#' pairs <- t(combn(rownames(sce_MOB_subset),2))
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#'
#' testingScaffold(scHOT_spatial) <- pairs
#'
#' @return A scHOT object
#'
#' @export
`testingScaffold<-` = function(x, value) {
  x@testingScaffold <- value
  x
}


#' Setter functions for scHOT objects
#'
#' @title Setter functions for scHOT objects
#' @param x A scHOT object
#' @param value The value the slot should take
#'
#' @examples
#' 
#'library(SingleCellExperiment)
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#'
#' W <- spatialWeightMatrix(colData(scHOT_spatial)[,slot(scHOT_spatial, "positionColData")])
#'
#' weightMatrix(scHOT_spatial) <- W
#'
#' @return A scHOT object
#'
#' @export
`weightMatrix<-` = function(x, value) {
  x@weightMatrix <- value
  x
}

#' Setter functions for scHOT objects
#'
#' @title Setter functions for scHOT objects
#' @param x A scHOT object
#' @param value The value the slot should take
#'
#' @examples
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#'
#' scHOT_output(scHOT_spatial) <- data.frame()
#'
#' @return A scHOT object
#'
#' @export
`scHOT_output<-` = function(x, value) {
  x@scHOT_output <- value
  x
}

#' Setter functions for scHOT objects
#'
#' @title Setter functions for scHOT objects
#' @param x A scHOT object
#' @param value The value the slot should take
#'
#' @examples
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#'
#' params = list(higherOrderFunction = weightedSpearman,
#' higherOrderFunctionType = "weighted")
#'
#' params(scHOT_spatial) <- params
#'
#' @return A scHOT object
#'
#' @export
`params<-` = function(x, value) {
  x@params <- value
  x
}

#' Setter functions for scHOT objects
#'
#' @title Setter functions for scHOT objects
#' @param x A scHOT object
#' @param value The value the slot should take
#'
#' @examples
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#'
#' positionType(scHOT_spatial) <- "spatial"
#'
#' @return A scHOT object
#'
#' @export
`positionType<-` = function(x, value) {
  x@positionType <- value
  x
}

#' Setter functions for scHOT objects
#'
#' @title Setter functions for scHOT objects
#' @param x A scHOT object
#' @param value The value the slot should take
#'
#' @examples
#'
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#' assayName = "logcounts",
#' positionType = "spatial",
#' positionColData = c("x", "y"))
#'
#' positionColData(scHOT_spatial) <- c("x", "y")
#'
#' @return A scHOT object
#'
#' @export
`positionColData<-` = function(x, value) {
  x@positionColData <- value
  x
}

#' A wrapper function to perform scHOT
#'
#' @title scHOT
#'
#' @param scHOT A scHOT object
#' @param testingScaffold A matrix with rows for each testing combination
#' @param weightMatrix A matrix indicates the weight matrix for scHOT analysis
#' @param positionType A string indicating the position type,
#' either "trajectory" or "spatial"
#' @param positionColData Either trajectory or spatial information
#' for each sample. If positionType is "trajectory"
#' then positionColData should be a character or numeric indicating
#' the subset of colData of the scHOT object.
#' If positionType is "spatial" then positionColData should be
#' a character or numeric vector indicating the subset of colData that
#' give the full spatial coordinates.
#' @param nrow.out The number of weightings to include for testing,
#' a smaller value is faster for computation
#' @param averageAcrossTrajectoryTies Logical indicating whether ties
#' in the trajectory should be given the same local weights
#' @param higherOrderFunction A function object indicates the
#' higher order function
#' @param higherOrderFunctionType is "weighted" or "unweighted",
#' determines if there
#' is a weighting argument in the higher order function
#' @param numberPermutations The number of permutations,
#' set as 1000 by default
#' @param numberScaffold The number of testing scaffolds to
#'  perform permutations, set as 100 by default
#' @param storePermutations  a logical flag on whether
#' permutation values should be saved
#' @param higherOrderSummaryFunction A functon indicating the higher order
#' summary function (default is standard deviation `sd`)
#' @param parallel A logical input indicating whether to run
#' the permutation test using multiple cores in parallel.
#' @param BPPARAM  A \code{BiocParallelParam} class object from
#' the \code{BiocParallel} package is used. Default is SerialParam().
#' @param usenperm_estimate Logical (default FALSE) if number of neighbouring
#' permutations should be used to estimate P-values, or
#' if difference of global higher order statistic should be used
#' @param nperm_estimate Number of neighbouring permutations to
#' use for p-value estimation
#' @param maxDist max difference of global higher order statistic to
#' use for p-value estimation (default 0.1)
#' @param plot A logical input indicating whether the results are plotted
#' @param verbose A logical input indicating whether the intermediate
#' steps will be printed
#'
#' @param ... parameters for function trajectoryWeightMatrix or spatialWeightMatrix
#'
#' @return A scHOT object
#'
#' @examples
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                     assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                     positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#'
#' scHOT_spatial <- scHOT(scHOT_spatial,
#'                        testingScaffold = pairs,
#'                        positionType = "spatial",
#'                        positionColData = c("x", "y"),
#'                        nrow.out = NULL,
#'                        higherOrderFunction = weightedSpearman,
#'                        higherOrderFunctionType = "weighted",
#'                        numberPermutations = 100,
#'                        higherOrderSummaryFunction = sd,
#'                        parallel = FALSE,
#'                        verbose = TRUE,
#'                        span = 0.05)
#' @importFrom stats sd
#'
#' @export


scHOT <- function(scHOT,
                  testingScaffold = NULL,
                  weightMatrix = NULL,
                  positionType = NULL,
                  positionColData = NULL,
                  nrow.out = NULL,
                  averageAcrossTrajectoryTies = FALSE,
                  higherOrderFunction = NULL,
                  higherOrderFunctionType = NULL,
                  numberPermutations = 1000,
                  numberScaffold = 100,
                  storePermutations = TRUE,
                  higherOrderSummaryFunction = sd,
                  parallel = FALSE,
                  BPPARAM = BiocParallel::SerialParam(),
                  usenperm_estimate = FALSE,
                  nperm_estimate = 10000,
                  maxDist = 0.1,
                  plot = FALSE,
                  verbose = TRUE,
                  ...
) {


  # add testing scaffold


  # if (ncol(testingScaffold) != 2) {
  #   stop("testingScaffold must be a matrix with two columns \n")
  # }

  # rownames(testingScaffold) <- paste(testingScaffold[, 1], testingScaffold[, 2])
  # if (nrow(testingScaffold) == 1) {
  # rownames(testingScaffold) <- apply(testingScaffold,1,paste0, collapse = "_")
  # } else {
  rownames(testingScaffold) <- apply(testingScaffold, 1, paste, collapse = "_")
  # }

  if (verbose) {
    cat("Adding testing scaffold \n")
  }

  scHOT <- scHOT_addTestingScaffold(scHOT,
                                    testingScaffold = testingScaffold)

  # set weight matrix

  if (verbose) {
    cat("Set weight matrix \n")
  }

  scHOT <- scHOT_setWeightMatrix(scHOT,
                                 weightMatrix = weightMatrix,
                                 positionColData = positionColData,
                                 positionType = positionType,
                                 nrow.out = nrow.out,
                                 averageAcrossTrajectoryTies =
                                   averageAcrossTrajectoryTies,
                                 ...)

  if (verbose) {
    cat("Calculate gobal higher order function \n")
  }

  scHOT <- scHOT_calculateGlobalHigherOrderFunction(scHOT,
                                                    higherOrderFunction =
                                                      higherOrderFunction,
                                                    higherOrderFunctionType =
                                                      higherOrderFunctionType)


  if (verbose) {
    cat("Calculate Higher Order Test Statistics \n")
  }

  scHOT <- scHOT_setPermutationScaffold(scHOT,
                                        numberPermutations = numberPermutations,
                                        numberScaffold = numberScaffold,
                                        storePermutations = storePermutations)




  scHOT <- scHOT_calculateHigherOrderTestStatistics(scHOT,
                                                    higherOrderSummaryFunction =
                                                      higherOrderSummaryFunction)

  if (verbose) {
    cat("Perform Permutation Test \n")
  }




  scHOT <- scHOT_performPermutationTest(scHOT,
                                        verbose = verbose,
                                        parallel = parallel,
                                        BPPARAM = BPPARAM)

  if (storePermutations) {

    if (verbose) {
      cat("Estimating p-values \n")
    }

    scHOT <- scHOT_estimatePvalues(scHOT,
                                   usenperm_estimate = usenperm_estimate,
                                   nperm_estimate = nperm_estimate,
                                   maxDist = maxDist,
                                   plot = plot,
                                   verbose = verbose)
  } else {
    if (verbose) {
      cat("No permutations stored, not estimating p-values \n")
    }
  }

  return(scHOT)
}

##############################################

#' the plotColouredExpression function plots an n-panel scatterplot
#' of the gene pairs split by early, mid, and late in the sample ordering.
#'
#' @title plotColouredExpression
#'
#' @param scHOT A scHOT object.
#' @param genes is either a single character string with a delimeter,
#' or a length two character vector
#' @param genes_delimeter is the delimeter to split into two gene names
#' if genes is provided as a single character
#' @param branches A character indicates that the colnames stored
#' the branch information in colData
#' @param ranked_by A character indicates that the colnames stored
#' the ranking information of the cells in colData, such as trajectory time,
#' if it is NULL, it will be ranked based on the branch information.
#' @param subsetBranch subsetBranch is a character vector containing
#' the names of the branches to be plotted. If NULL it will plot all branches
#' @param n number of panels to split ranked samples into, default 3.
#' @param fittedline logical default TRUE, add a lm straight line to the plot
#' @param assayName the name of the assay that are used to plot.
#' @return \code{ggplot} a ggplot object of scatterplots of expression
#' split by sample ordering
#'
#' @examples
#'
#' data(liver)
#'
#' scHOT_traj <- scHOT_buildFromMatrix(
#' mat = liver$liver_branch_hep,
#' cellData = list(pseudotime = liver$liver_pseudotime_hep),
#' positionType = "trajectory",
#' positionColData = "pseudotime")
#'
#' scHOT_traj
#'
#' plotColouredExpression(scHOT_traj, c("Cdt1","Top2a"), n = 5)
#'
#' @importFrom SummarizedExperiment assay colData
#' @import ggplot2
#'
#'
#' @export

plotColouredExpression = function(scHOT,
                                  genes,
                                  genes_delimeter = "_",
                                  branches = NULL,
                                  ranked_by = NULL,
                                  subsetBranch = NULL,
                                  n = 3,
                                  fittedline = TRUE,
                                  assayName = NULL) {

  if (length(genes) == 0) stop("please provide genes to plot")

  if (length(genes) == 1) {
    message("genes given as single character, splitting using delimiter")
    genepair = unlist(strsplit(genes, genes_delimeter))[seq_len(2)]
  }

  if (length(genes) == 2) {
    genepair = genes
  }

  if (length(genes) > 2) {
    message("only first two entries of genes will be used")
    genepair = genes[seq_len(2)]
  }


  if (is.null(assayName)) {
    assayName <- "expression"
  }

  branchData <- SummarizedExperiment::assay(scHOT, assayName)

  if (is.null(ranked_by)) {
    message("ranked_by information is not provided,
            the expression data is ranked by the branches")
  } else {

    if (!(ranked_by %in% colnames(SummarizedExperiment::colData(scHOT)))) {
      stop("ranked_by provided is not found in colData(scHOT)")
    }

    branchData <- branchData[, order(colData(scHOT)[, ranked_by])]
  }



  if (is.null(branches)) {
    message("branches information is not provided")
    branchData = list(Branch = branchData)
  } else {
    if (!(branches %in% colnames(colData(scHOT)))) {
      stop("branches provided is not found in colData(scHOT)")
    }

    branch_info <- SummarizedExperiment::colData(scHOT)[, branches]

    branchData <- lapply(split(seq_along(branch_info), branch_info),
                         function(idx)
                           branchData[, idx])[order(unique(branch_info))]
  }



  if (!all(unlist(lapply(branchData, function(x) genepair %in% rownames(x))))) {
    stop("At least one gene name not exists in the expression data")
  }



  gdf = do.call(rbind, lapply(branchData, function(branch) {
    gdf_list_1 = data.frame(Sample = colnames(branch),
                            order = seq_len(ncol(branch)),
                            ExpressionGene1 = branch[genepair[1], ],
                            ExpressionGene2 = branch[genepair[2], ])
    if (n > 1) {


      g_labels <- unlist(ifelse(n == 3,
                                list(c("Early", "Middle",
                                       "Late")),
                                ifelse(n == 2, list(c("Early",
                                                      "Late")),
                                       list(paste0("Group ",
                                                   seq_len(n))))))
      gdf_list_1$ordercut = cut(gdf_list_1$order, n, labels = g_labels)
    } else {
      gdf_list_1$ordercut = rep("All cells", times = nrow(gdf_list_1))
    }
    return(gdf_list_1)
  }))

  gdf$branch = rep(names(branchData), times = unlist(lapply(branchData,
                                                            ncol)))
  if (!is.null(subsetBranch)) {
    gdf_sub = subset(gdf, gdf$branch %in% subsetBranch)
    if (nrow(gdf_sub) == 0)
      stop("no branches with names in subsetBranch,
           please re-run with correct names (should match names of branchData)")
  }
  else {
    gdf_sub = gdf
  }
  g = ggplot2::ggplot(gdf_sub, aes(x = gdf_sub$ExpressionGene1,
                                   y = gdf_sub$ExpressionGene2,
                                   color = order)) +
    geom_point(show.legend = FALSE, alpha = 0.7) +
    facet_grid(branch ~ ordercut, scales = "free_y") +
    scale_color_gradientn(colours = c("orange", "blue")) +
    xlab(genepair[1]) + ylab(genepair[2]) + theme_classic() +
    NULL
  if (fittedline) {
    g = g +
      geom_smooth(colour = "black", fill = NA, linetype = "dashed",
                  method = "lm") +
      NULL
  }
  return(g)
}



##############################################

#' the plotOrderedExpression function plots expression vectors
#' along branches and genes as ribbon plots
#'
#' @title plotOrderedExpression
#' @param scHOT A scHOT object, where the expression
#' data is stored in the assay slot, with assay name "expression".
#' @param genes is a character vector for gene names
#' @param positionType A string indicates the position type,
#' either trajectory or spatial
#' @param branches A character indicates that the colnames stored
#' the branch information in colData
#' @param ranked_by A character indicates that the colnames stored
#' the ranking information of the cells in colData, such as trajectory time
#' @param xvals A character indicates that the colnames stored in
#' colData of the x-values associated with the samples in branchData
#' @param subsetBranch subsetBranch is a character vector containing
#' the names of the branches to be plotted. If NULL it will plot all branches
#' @param facet can either be FALSE, "branch", "gene", or "both"
#' @param positionColData A vector indicates column names of colData that
#' stored the postion informaton (for spatial type of data)
#' @param assayName the name of the assay that are used to plot.
#'
#' @return \code{ggplot} a ggplot object for ribbon plot with points
#'
#'
#' @importFrom SummarizedExperiment assay colData
#' @import ggplot2
#' @importFrom reshape melt
#'
#' @export
#'
plotOrderedExpression = function(scHOT,
                                 genes,
                                 positionType = NULL,
                                 branches = NULL,
                                 ranked_by = NULL,
                                 xvals = NULL,
                                 subsetBranch = NULL,
                                 facet = FALSE,
                                 positionColData = NULL,
                                 assayName = NULL) {


  # genes is character vector
  # xvals is a list containing the x-values associated with the
  # samples in branchData
  # (if NULL, samples will just be plotted against their rank)
  # subsetBranch is a character vector containing the names of
  # the branches to be plotted. If NULL it will plot all branches
  # facet can either be FALSE, "branch", "gene", or "both"


  if (is.null(positionType)) {

    if (is.null(scHOT@positionType)) {
      stop("Both positionType and scHOT@positionType are NULL.")
    } else {
      positionType <- scHOT@positionType
    }

  }

  # if (length(genepair) == 1) {
  #   genepair = unlist(strsplit(genepair, "_"))
  # }
  # else {
  #   genepair = genepair[1:2]
  # }

  if (is.null(assayName)) {
    assayName <- "expression"
  }

  branchData <- SummarizedExperiment::assay(scHOT, assayName)

  genes = genes[genes %in% rownames(branchData)]

  if (length(genes) == 0) {
    stop("No genes found in rownames of dataset!")
  }

  if (scHOT@positionType == "trajectory") {


    if (is.null(ranked_by)) {
      message("ranked_by information is not provided,
              the expression data is ranked by the branches")
    } else {

      if (!(ranked_by %in% colnames(SummarizedExperiment::colData(scHOT)))) {
        stop("ranked_by provided is not found in colData(scHOT)")
      }

      branchData <- branchData[, order(colData(scHOT)[, ranked_by])]
    }



    if (is.null(branches)) {
      message("branches information is not provided")
      branchData = list(Branch = branchData)
    } else {
      if (!(branches %in% colnames(colData(scHOT)))) {
        stop("branches provided is not found in colData(scHOT)")
      }

      branch_info <- SummarizedExperiment::colData(scHOT)[, branches]

      branchData <- lapply(split(seq_along(branch_info), branch_info),
                           function(idx)
                             branchData[, idx])[order(unique(branch_info))]
    }


    gdf_list = sapply(genes, function(g) {

      gdf = do.call(rbind,lapply(branchData, function(branch){
        gdf_list_1 = data.frame(
          Sample = colnames(branch),
          order = seq_len(ncol(branch)),
          ExpressionGene = branch[g,],
          gene = g
        )
        return(gdf_list_1)
      }))
      gdf$branch = rep(names(branchData),
                       times = unlist(lapply(branchData, ncol)))
      return(gdf)
    }, simplify = FALSE)

    gdf = do.call(rbind, gdf_list)

    if (!is.null(subsetBranch)) {
      gdf_sub = subset(gdf, gdf$branch %in% subsetBranch)
      if (nrow(gdf_sub) == 0) stop("no branches with names in subsetBranch,
                                 please re-run with correct names
                                   (should match names of branchData)")
    } else {
      gdf_sub = gdf
    }


    if (!is.null(xvals)) {

      if (!(xval %in% colnames(SummarizedExperiment::colData(scHOT)))) {
        stop("xval provided is not found in colData(scHOT)")
      }
      xval <- lapply(split(seq_along(branch_info), branch_info),
                     function(idx) xval[, idx])[order(unique(branch_info))]
      xval <- apply(as.matrix(gdf_sub), 1,
                    function(x)xvals[[x["branch"]]][as.numeric(x["order"])])
      gdf_sub$order <- xval

    }

    g = ggplot(gdf_sub, aes(x = order, y = gdf_sub$ExpressionGene,
                            colour = gdf_sub$gene,
                            fill = gdf_sub$gene, linetype = gdf_sub$branch,
                            shape = gdf_sub$branch)) +
      geom_point() +
      labs(fill = "Gene", col = "Gene", linetype = "Branch", shape = "Branch") +
      theme_classic() + geom_smooth() +
      ylab("Expression") +
      ggtitle(paste0(genes, collapse = ", ")) +
      NULL

    if (facet == "branch") {
      g = g + facet_grid(~branch)
    }

    if (facet == "gene") {
      g = g + facet_grid(gene~.)
    }

    if (facet == "both") {
      g = g + facet_grid(gene~branch)
    }
  }

  if (scHOT@positionType == "spatial") {

    if (is.null(positionColData)) {

      if (is.null(scHOT@positionColData)) {
        stop("Both positionColData and scHOT@positionColData are NULL.")
      } else {
        positionColData <- scHOT@positionColData
      }

    }

    coords_info <- data.frame(SummarizedExperiment::colData(scHOT)[, positionColData])
    colnames(coords_info) <- positionColData

    branch_long <- reshape::melt(cbind(coords_info,
                                       t(branchData[genes, , drop = FALSE])),
                                 id.vars = positionColData)


    colnames(branch_long) <- c("x", "y", "genes", "value")

    g <- ggplot(branch_long,  aes(x = branch_long$x,
                                  y = branch_long$y,
                                  color = branch_long$value)) +
      geom_point(size = 3) +
      # geom_point(size = 0.5, colour = "black") +
      theme_classic() +
      facet_wrap(~genes) +
      scale_alpha_continuous(range = c(0,0.5)) +
      scale_color_viridis_c(breaks = c(0,max(branch_long$value)),
                            limits = c(0,max(branch_long$value)),
                            labels = c("Low","High")) +
      theme(panel.grid = element_blank()) +
      theme(axis.ticks = element_blank()) +
      theme(axis.text = element_blank()) +
      xlab("") + ylab("") +
      coord_fixed() +
      labs(col = "Expression value") +
      NULL
  }




  return(g)
}


##############################################

#' the plotEgoNetwork function plots network graphs with edges coloured by
#' weights in the network
#'
#' @title plotEgoNetwork
#' @param scHOT a scHOT object
#' @param hubnode is a character vector of node(s) to include as hub nodes
#' @param network is an igraph network
#' @param weight A string indicates the column name stored in scHOT_output slot
#' that are used as the weights of the network
#' @param subset is a logical asking if you should subset
#' based on the weight (default FALSE)
#' @param thresh is the subset weight threshold
#' @return \code{igraph} object containing the network graphed.
#' Produces an igraph plot
#'
#'
#' @import igraph
#' @importFrom graphics plot
#'
#' @export


plotEgoNetwork = function(scHOT, hubnode, network,
                          weight = "higherOrderStatistic",
                          subset = FALSE,
                          thresh = NULL) {


  # hubnode is a character vector of node(s) to include as hub nodes
  # g is an igraph network, with E(g)[[weight]] given as DCARS test statistics
  # weight is a character vector containing edge weights,
  # associated with different branches
  # subset is a logical asking if you should subset based on the weight
  # thresh is the subset weight threshold

  if (is.null(weight)) {
    igraph::E(network)$weight <- rep(1, ncol(scHOT))
  } else {
    if (!(weight %in% colnames(scHOT@scHOT_output))) {
      stop("weight provided is not found in scHOT_output")
    }

    igraph::E(network)$weight <- scHOT@scHOT_output[, weight]
  }

  if (!hubnode %in% names(V(network))) {
    stop("The provided hubnode is not a node in the network")
  }


  nodes = unique(names(unlist(igraph::neighborhood(network, nodes = hubnode))))
  subego = igraph::induced.subgraph(network, vids = nodes)
  subego = igraph::simplify(subego, edge.attr.comb = "mean")

  if (subset) {
    if (!all(weight %in% names(edge_attr(subego)))) {
      stop("at least one weight missing from edge attributes,
           either re-specify weights or rerun with subset = FALSE")
    }
    if (is.null(thresh)) {
      message("no threshold given, using 0 as default")
      thresh = 0
    }
    keepedges <- apply(sapply(weight,
                              function(w)
                                igraph::edge_attr(subego)[[w]] > thresh,
                              simplify = TRUE), 1, any)
    subego <- igraph::subgraph.edges(subego, which(keepedges))
  }

  igraph::V(subego)$color = "beige"
  igraph::V(subego)$label.color = "black"
  igraph::V(subego)$label.family = "sans"
  igraph::V(subego)$label.cex = 0.7
  igraph::V(subego)$size = 20
  igraph::V(subego)$frame.color = "black"
  igraph::V(subego)$frame.size = 5
  lyout = igraph::layout.davidson.harel(subego)
  width = 5

  maxval = ceiling(max(50*unlist(edge_attr(subego)[["weight"]])))
  colvals = grDevices::colorRampPalette(c("grey","red"))(maxval)

  graphics::plot(subego, layout = lyout,
                 edge.color = colvals[ceiling(50*edge_attr(subego)[["weight"]])],
                 edge.width = width,
                 # xlab = "weight",
                 main = hubnode)

  # par(mfrow = c(1,length(weight)))
  #
  # for (i in weight) {
  #   plot(subego, layout = lyout,
  #        edge.color = colvals[ceiling(50*edge_attr(subego)[["weight"]])],
  #        edge.width = width,
  #        xlab = i,
  #        main = hubnode)
  # }
  return(subego)
}


##############################################


#' the plotHigherOrderSequence function plots weighted higher order
#' statistic vectors (stored in higherOrderSequence) as line plots
#'
#' @title plotHigherOrderSequence
#' @param scHOT A scHOT object with higherOrderSequence in scHOT_output slot
#' @param gene is either a logical vector matching rows of entries
#' in wcorsList, or a character of a gene
#' @param positionType A string indicates the position type,
#' either trajectory or spatial
#' @param branches A character indicates that the colnames stored
#' the branch information in colData (for trajectory type of data)
#' @param positionColData A vector indicates column names of colData
#' that stored the postion informaton (for spatial type of data)
#' @return \code{ggplot} object with line plots
#'
#' @importFrom reshape melt
#' @import ggplot2
#' @importFrom ggforce geom_voronoi_tile
#'
#' @examples
#' data(liver)
#'
#' scHOT_traj <- scHOT_buildFromMatrix(
#' mat = liver$liver_branch_hep,
#' cellData = list(pseudotime = liver$liver_pseudotime_hep),
#' positionType = "trajectory",
#' positionColData = "pseudotime")
#' scHOT_traj
#' plotColouredExpression(scHOT_traj, c("Cdt1","Top2a"), n = 5)
#'
#' scHOT_traj <- scHOT_addTestingScaffold(scHOT_traj,
#' t(as.matrix(c("Cdt1", "Top2a"))))
#' scHOT_traj <- scHOT_setWeightMatrix(scHOT_traj,
#' positionColData = c("pseudotime"),
#' positionType = "trajectory",
#' nrow.out = NULL,
#' span = 0.25)
#' scHOT_traj <- scHOT_calculateGlobalHigherOrderFunction(scHOT_traj,
#'                                                       higherOrderFunction =
#'                                                       weightedSpearman,
#'                                                      higherOrderFunctionType =
#'                                                      "weighted")
#'
#' scHOT_traj <- scHOT_calculateHigherOrderTestStatistics(scHOT_traj,
#'                                                       higherOrderSummaryFunction =
#'                                                       sd)
#'
#' slot(scHOT_traj, "scHOT_output")
#' plotHigherOrderSequence(scHOT_traj, c("Cdt1_Top2a"))
#'
#' @export
#'
plotHigherOrderSequence <- function(scHOT,
                                    gene,
                                    positionType = NULL,
                                    branches = NULL,
                                    positionColData = NULL) {

  # matchExact matches gene names by splitting
  #instead of using grep, but is slower

  if (!("higherOrderSequence" %in% colnames(scHOT@scHOT_output))) {
    stop("higherOrderSequence is not found in scHOT_output")
  }

  namescols = grep("gene_", colnames(scHOT@scHOT_output), value = TRUE)

  wcor <- as.matrix(scHOT@scHOT_output$higherOrderSequence)
  # rownames(wcor) <- paste0(scHOT@scHOT_output$gene_1,
  # scHOT@scHOT_output$gene_2, sep = "_")
  rownames(wcor) <- apply(scHOT@scHOT_output[,namescols, drop = FALSE],
                          1, paste0, collapse = "_")
  colnames(wcor) <- NULL

  if (ncol(wcor) != ncol(scHOT)) {
    warning("Not all the cell position has higherOrderSequence statistics,
            set nrow.out = NULL in scHOT_setWeightMatrix to calculate
            higherOrderSequence for all positions!")
  }

  if (is.null(positionType)) {

    if (is.null(scHOT@positionType)) {
      stop("Both positionType and scHOT@positionType are NULL.")
    } else {
      positionType <- scHOT@positionType
    }

  }

  positionType <- match.arg(positionType, c("trajectory","spatial"),
                            several.ok = FALSE)




  if (positionType == "trajectory") {

    if (is.null(branches)) {
      message("branches information is not provided")
      wcorsList = list(Branch = wcor)
    } else {
      if (!(branches %in% colnames(colData(scHOT)))) {
        stop("branches provided is not found in colData(scHOT)")
      }

      branch_info <- SummarizedExperiment::colData(scHOT)[, branches]

      wcorsList <- lapply(split(seq_along(branch_info), branch_info),
                          function(idx) wcor[, idx])[order(unique(branch_info))]
    }



    if (is.null(names(wcorsList))) {
      names(wcorsList) <- paste0("Branch_", seq_len(length(wcorsList)))
    }

    if (is.logical(gene[1])) {

      if (length(unique(unlist(lapply(wcorsList,nrow)))) > 1) {
        stop("cannot use logical subset when weighted higher
             order statistic matrices have differing rows")
      }

      if (length(gene) != nrow(wcorsList[[1]])) {
        stop("cannot use logical subset when length of gene
             doesn't match nrow of wcorsList matrices")
      }

      wcors_longList = lapply(wcorsList,function(branch){
        reshape::melt(t(branch[gene,]))
      })

      gene = ""
    } else {

      gene = paste0(sort(gene), collapse = "|")

      wcors_longList = lapply(wcorsList,function(branch){
        reshape::melt(t(branch[grepl(gene, rownames(branch)), , drop = FALSE]))
      })

    }

    branch_long = do.call(rbind, wcors_longList)
    branch_long = cbind(
      rep(names(wcors_longList), unlist(lapply(wcors_longList, nrow))),
      branch_long)
    colnames(branch_long) = c("branch","SampleOrder",
                              "GenePair", "WeightedCorrelation")



    if (max(abs(branch_long$WeightedCorrelation)) < 1) {
      ylimit <- ylim(c(-1, 1))
    } else {
      ylimit <- NULL
    }

    g <- ggplot(branch_long,
                aes(x = branch_long$SampleOrder,
                    y = branch_long$WeightedCorrelation,
                    group = branch_long$GenePair,
                    col = branch_long$GenePair)) +
      geom_line(size = 2, alpha = 0.6) +
      facet_grid(~branch, scales = "free_x") +
      theme_classic() +
      ylimit +
      geom_hline(yintercept = 0, size = 1, colour = "grey") +
      ggtitle(gene) +
      xlab("Sample Order") +
      ylab("Weighted Higher Order Statistic") +
      labs(col = "Test") +
      NULL


  }


  # spatial case

  if (positionType == "spatial") {
    if (is.logical(gene[1])) {

      if (length(unique(unlist(lapply(wcorsList, nrow)))) > 1) {
        stop("cannot use logical subset when weighted higher
             order statistic matrices have differing rows")
      }

      if (length(gene) != nrow(wcor)) {
        stop("cannot use logical subset when length of gene
             doesn't match nrow of wcorsList matrices")
      }


      gene = ""
    } else {


      if (!all(gene %in% rownames(wcor))) {

        if (length(gene) == 2) {
          if (!paste0(sort(gene), collapse = "_") %in% rownames(wcor)) {
            stop("gene pairs has no higherOrderSequence ")
          } else {
            gene <- paste0(sort(gene), collapse = "_")
          }
        } else {
          stop("gene pairs has no higherOrderSequence ")
        }
      }
    }

    if (is.null(positionColData)) {

      if (is.null(scHOT@positionColData)) {
        stop("Both positionColData and scHOT@positionColData are NULL.")
      } else {
        positionColData <- scHOT@positionColData
      }

    }

    coords_info <- data.frame(colData(scHOT)[, positionColData])
    colnames(coords_info) <- positionColData

    wcor_all <- matrix(NA, nrow = length(gene), ncol = ncol(scHOT))
    rownames(wcor_all) <- gene
    colnames(wcor_all) <- seq_len(ncol(scHOT))
    # colnames(wcor_all) <- colnames(scHOT)
    wcor_all[gene, rownames(scHOT@weightMatrix)] <- wcor[gene, , drop = FALSE]


    branch_long <- reshape::melt(cbind(coords_info, t(wcor_all)),
                                 id.vars = positionColData)

    colnames(branch_long) <- c("x", "y", "genepair", "value")

    g <- ggplot(branch_long,  aes(x = branch_long$x,
                                  y = branch_long$y,
                                  fill = branch_long$value)) +
      ggforce::geom_voronoi_tile(max.radius = 1) +
      geom_point(size = 0.5, colour = "black") +
      theme_classic() +
      facet_wrap(~genepair) +
      scale_alpha_continuous(range = c(0,0.5)) +
      scale_fill_gradient2(low = "blue", mid = "white",
                           high = "red", limits = c(-1,1)) +
      theme(panel.grid = element_blank()) +
      theme(axis.ticks = element_blank()) +
      theme(axis.text = element_blank()) +
      xlab("") + ylab("") +
      coord_fixed() +
      labs(fill = "Weighted Higher Order Statistic") +
      NULL
  }

  return(g)
}



##############################################

#' the scHOT_plotPermutationDistributions function plots
#' the permutation test statistics as a diagnostic plot
#' for estimating p-values
#'
#' @title scHOT_plotPermutationDistributions
#' @param scHOT a scHOT object
#' @return \code{ggplot} graph of global higher order function and the
#' permutation scHOT test statistics. This should have a continuous pattern
#' to be reliably used for p-value estimation
#'
#'
#' @import ggplot2
#' @importFrom ggplot2 ggplot
#' @importFrom stats loess quantile
#'
#' @export

scHOT_plotPermutationDistributions = function(scHOT) {

  scHOT_output = scHOT@scHOT_output

  if (length(scHOT_output$globalHigherOrderFunction) == 0) {
    stop("No globalHigherOrderFunction found in
    scHOT object's scHOT_output slot,
         please run function scHOT_calculateGlobalHigherOrderFunction!")
  }

  if (length(as.list(scHOT_output$permutations)) == 0) {
    stop("No permutations found in scHOT object's scHOT_output slot,
         please run function scHOT_performPermutationTest!")
  }

  permstatsDF = data.frame(
    test = rep(seq_len(nrow(scHOT_output)),
               times = unlist(
                 lapply(scHOT_output$permutations,
                        function(x) length(unlist(x))))),
    stat = unlist(scHOT_output$permutations),
    globalHigherOrderFunction = rep(scHOT_output$globalHigherOrderFunction,
                                    times = unlist(
                                      lapply(scHOT_output$permutations,
                                             function(x) length(unlist(x)))))
  )

  quantileDF = data.frame(
    test = rep(seq_len(nrow(scHOT_output)),
               times = lapply(scHOT_output$permutations,
                              function(x) length(unlist(x)) > 0)),
    quantile_0.9 = unlist(lapply(scHOT_output$permutations,
                                 function(x) stats::quantile(x, 0.9,
                                                             na.rm = TRUE))),
    globalHigherOrderFunction = rep(scHOT_output$globalHigherOrderFunction,
                                    times = lapply(scHOT_output$permutations,
                                                   function(x) length(unlist(x)) > 0))
  )

  quantileDF$quantile_0.9_fitted <- NA
  quantileDF$quantile_0.9_fitted[!is.na(quantileDF$quantile_0.9)] =
    loess(quantile_0.9 ~ globalHigherOrderFunction, data = quantileDF)$fitted

  gBase = ggplot(permstatsDF, aes(x = permstatsDF$globalHigherOrderFunction,
                                  y = permstatsDF$stat))


  if (requireNamespace("scales", quietly = TRUE) &
      requireNamespace("scattermore", quietly = TRUE)) {
    gBase = gBase + scattermore::geom_scattermore()
  }


  df_toPlot <- reshape::sort_df(subset(quantileDF,
                                       !is.na(quantileDF$quantile_0.9_fitted)),
                                "globalHigherOrderFunction")
  g = gBase +
    geom_line(aes(x = df_toPlot$globalHigherOrderFunction,
                  y = df_toPlot$quantile_0.9_fitted),
              data = df_toPlot) +
    theme_classic() +
    ylab("Permuted scHOT test statistics") +
    xlab("globalHigherOrderFunction") +
    NULL


  return(g)
}

#' @title Liver trajectory example data
#' @description A liver data set contains two branches
#' (hepatocyte and cholangiocyte).
#' @usage data(liver, package = 'scHOT')
#'
#' @format A `list` object.
"liver"

#' @title MOB_subset spatial example data
#' @description  A MOB spatial data set.
#' @usage data(MOB_subset, package = 'scHOT')
"MOB_subset"

#' @importClassesFrom Matrix dgCMatrix
#'

setClassUnion("matrixORdgCMatrix", c("matrix", "dgCMatrix"))

#' @importClassesFrom S4Vectors DataFrame DFrame
#'

setClassUnion("data.frameORDataFrame", c("data.frame", "DataFrame"))


setClassUnion("characterORNULL", c("character", "NULL"))



check_validity <- function(object) {

  if (!is.null(object@positionType)) {
    if (!object@positionType %in% c("trajectory", "spatial") |
        length(object@positionType) > 1) {
      errors <- c("positionType should be either trajectory or spatial")
      return(errors)
    }
  }

  if (!is.null(object@positionColData)) {
    if (!all(object@positionColData %in% colnames(colData(object)))) {
      errors <- c("positionColData should be one of the colnames of
                  colData of the scHOT object")
      return(errors)
    }
  }

  return(TRUE)

}


#' scHOT class
#'
#' @slot testingScaffold A matrix with rows for each testing combination
#' @slot weightMatrix A matrix or dgCMatrix indicates the weight matrix
#' @slot scHOT_output A data.frame or DtatFrame to store output from scHOT
#' @slot params A list of parameters
#' @slot positionType A character indicates the type of the position,
#' either trajectory or spatial
#' @slot positionColData A vector indicates column names of colData
#' that stored the postion informaton
#'
#' @importClassesFrom SingleCellExperiment SingleCellExperiment
#' @importFrom methods new
#' @export

.scHOT <- setClass("scHOT",
                   slots = c(testingScaffold = "matrix",
                             weightMatrix = "matrixORdgCMatrix",
                             scHOT_output = "data.frameORDataFrame",
                             params = "list",
                             positionType = "characterORNULL",
                             positionColData = "characterORNULL"),
                   contains = "SingleCellExperiment")


#' @importFrom S4Vectors coolcat
#' @importFrom methods callNextMethod
#'
#'

setMethod("show", "scHOT", function(object) {
  methods::callNextMethod()
  cat("testingScaffold dim:", dim(object@testingScaffold), "\n")
  cat("weightMatrix dim:", dim(object@weightMatrix), "\n")
  S4Vectors::coolcat("scHOT_output colnames (%d): %s\n",
                     colnames(object@scHOT_output))
  S4Vectors::coolcat("param names (%d): %s\n", names(object@params))
  cat("position type:", as.character(object@positionType),"\n")
})


#' @importFrom S4Vectors setValidity2
#'
setValidity2("scHOT", check_validity)

###################################################################


#' Calculate and store the higherOrderSequence and higherOrderTestStatistic
#'
#' @title scHOT_calculateHigherOrderTestStatistics
#'
#' @param scHOT A scHOT object
#' @param higherOrderSummaryFunction A function which indicates how the higher
#' order sequence is summarised, default is sd
#' @param ... parameters for higherOrderSummaryFunction
#'
#'
#' @importFrom SummarizedExperiment assay
#' @importFrom IRanges NumericList
#'
#' @return scHOT A scHOT object with results stored in scHOT_output slot
#'
#' @examples
#' data(MOB_subset)
#' sce_MOB_subset <- MOB_subset$sce_MOB_subset
#' scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#'
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#' scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#'
#' scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
#'                                        positionColData = c("x","y"),
#'                                         positionType = "spatial",
#'                                         nrow.out = NULL,
#'                                         span = 0.05)
#' scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
#'   scHOT_spatial,
#'   higherOrderFunction = weightedSpearman,
#'   higherOrderFunctionType = "weighted")
#' scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
#'                                               numberPermutations = 100)
#' scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(
#'   scHOT_spatial,
#'   higherOrderSummaryFunction = sd)
#' @export





scHOT_calculateHigherOrderTestStatistics <- function(
  scHOT,
  higherOrderSummaryFunction = stats::sd,
  ...) {
  # calculate and store the higherOrderSequence and higherOrderTestStatistic

  higherOrderFunctionType <- scHOT@params$higherOrderFunctionType
  if (is.null(higherOrderFunctionType)) {
    stop("No higherOrderFunctionType found in scHOT object's params slot,
    please provide one or run scHOT_calculateGlobalHigherOrderFunction!")
  }

  higherOrderFunction <- scHOT@params$higherOrderFunction
  if (is.null(higherOrderFunction)) {
    stop("No higherOrderFunction found in scHOT object's params slot,
    please provide one or run scHOT_calculateGlobalHigherOrderFunction!")
  }

  if (is.null(higherOrderSummaryFunction)) {
    higherOrderSummaryFunction = scHOT@params$higherOrderSummaryFunction
    if (is.null(higherOrderSummaryFunction)) {
      stop("No higherOrderSummaryFunction provided or stored in scHOT object's params slot, please provide one!")
    }
  } else {
    message("higherOrderSummaryFunction will replace any stored param")
    scHOT@params$higherOrderSummaryFunction = higherOrderSummaryFunction
  }

  testingScaffold <- scHOT@testingScaffold

  if (is.null(testingScaffold)) {
    stop("No testingScaffold found in scHOT,
         please provide one or run scHOT_addTestingScaffold function!")
  }

  expressionData = SummarizedExperiment::assay(scHOT, "expression")
  if (is.null(expressionData)) {
    stop("No expressionData found, please provide an assay(scHOT, \"expression\") slot")
  }

  weightMatrix = scHOT@weightMatrix
  if (is.null(weightMatrix)) {
    stop("No weightMatrix found in scHOT,
         please provide one or run scHOT_setWeightMatrix!")
  }

  if (higherOrderFunctionType == "unweighted") {
    higherOrderSequence = functionOverScaffoldWithWeights(
      testingScaffold,
      expressionData,
      higherOrderFunction,
      weightMatrix)
  } else {
    # for weighted
    higherOrderSequence = weightedFunctionOverScaffold(
      testingScaffold,
      expressionData,
      higherOrderFunction,
      weightMatrix)
  }

  if (nrow(scHOT@scHOT_output) == 0) {
    # scHOT@scHOT_output = DataFrame(testingScaffold)
    scHOT_output(scHOT) <- DataFrame(testingScaffold)
  }

  scHOT <- scHOT_stripOutput(scHOT, force = FALSE)

  if (is.null(nrow(higherOrderSequence))) {
    # when only one pair to test
    scHOT@scHOT_output$higherOrderSequence = split(
      higherOrderSequence, 1)
  } else {
    scHOT@scHOT_output$higherOrderSequence = split(
      higherOrderSequence, seq_len(nrow(higherOrderSequence)))
  }



  scHOT@scHOT_output$higherOrderSequence <- IRanges::NumericList(
    scHOT@scHOT_output$higherOrderSequence)

  scHOT@scHOT_output$higherOrderStatistic = unlist(
    lapply(scHOT@scHOT_output$higherOrderSequence,
           higherOrderSummaryFunction, ...))

  return(scHOT)
}



###################################################################


#' Perform permutation test
#'
#' @title scHOT_performPermutationTest
#'
#' @param scHOT A scHOT object
#' @param verbose A logical input indicates whether the intermediate steps
#' will be printed
#' @param parallel A logical input indicates whether run the permutation test
#' using multiple cores in parallel.
#' @param BPPARAM  A \code{BiocParallelParam} class object
#' from the \code{BiocParallel} package is used. Default is SerialParam().
#'
#' @importFrom SummarizedExperiment assay
#' @importFrom IRanges NumericList
#' @importFrom stats p.adjust na.omit
#' @importFrom BiocParallel bplapply SerialParam
#'
#' @return scHOT A scHOT object with results stored in scHOT_output slot
#'
#' @examples
#'  data(MOB_subset)
#'  sce_MOB_subset <- MOB_subset$sce_MOB_subset
#'  scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#' scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#'  scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
#'                                        positionColData = c("x","y"),
#'                                         positionType = "spatial",
#'                                         nrow.out = NULL,
#'                                         span = 0.05)
#' scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
#'   scHOT_spatial,
#'   higherOrderFunction = weightedSpearman,
#'   higherOrderFunctionType = "weighted")
#' scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
#'                                               numberPermutations = 100)
#' scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(
#'   scHOT_spatial,
#'   higherOrderSummaryFunction = sd)
#'
#' scHOT_spatial <- scHOT_performPermutationTest(
#'   scHOT_spatial,
#'   verbose = TRUE,
#'   parallel = FALSE)
#'
#' @export



scHOT_performPermutationTest <- function(
  scHOT,
  verbose = FALSE,
  parallel = FALSE,
  BPPARAM = BiocParallel::SerialParam()) {

  # do this in a for loop

  higherOrderFunctionType <- scHOT@params$higherOrderFunctionType
  if (is.null(higherOrderFunctionType)) {
    stop("No higherOrderFunctionType found in scHOT object's params slot,
    please provide one or run scHOT_calculateGlobalHigherOrderFunction!")
  }


  higherOrderFunction <- scHOT@params$higherOrderFunction
  if (is.null(higherOrderFunction)) {
    stop("No higherOrderFunction found in scHOT object's params slot,
    please provide one or run scHOT_calculateGlobalHigherOrderFunction!")  }


  higherOrderSummaryFunction <- scHOT@params$higherOrderSummaryFunction
  if (is.null(higherOrderSummaryFunction)) {
    stop("No higherOrderSummaryFunction provided or stored in scHOT object's params slot, please provide one!")
  }

  expressionData <- SummarizedExperiment::assay(scHOT, "expression")
  if (is.null(expressionData)) {
    stop("No expressionData found, please provide an assay(scHOT, \"expression\") slot")
  }

  weightMatrix = scHOT@weightMatrix
  if (is.null(weightMatrix)) {
    stop("No weightMatrix found in scHOT object,
         please provide one or run scHOT_setWeightMatrix function!")
  }



  DF <- scHOT@scHOT_output

  if (nrow(DF) == 0) {
    stop("Need scHOT@scHOT_output object")
  }

  if (any(DF$storePermutations)) {
    DF$permutations = split(rep(NA, nrow(DF)), seq_len(nrow(DF)))
  }

  DF$pvalPermutations = rep(NA, nrow(DF))

  for (i in seq_len(nrow(DF))) {

    niter = DF[i,"numberPermutations"]

    if (niter == 0) next

    if (verbose) {
      if (nrow(DF) > 100) {
        if (i == 1 | i %% 10 == 0 | i == nrow(DF)) {
          cat(paste0("Permutation testing combination ", i,
                     " of ", nrow(DF), "...\n"))
        }
      } else {
        cat(paste0("Permutation testing combination ", i,
                   " of ", nrow(DF), "...\n"))
      }
    }

    store = DF[i,"storePermutations"]
    obs = DF[i,"higherOrderStatistic"]

    scaffold = scHOT@testingScaffold[i,,drop = FALSE]

    exprs = expressionData[scaffold[1,],,drop = FALSE]
    nc = ncol(exprs)


    if (parallel) {

      permutations <- BiocParallel::bplapply(seq_len(niter), function(i) {

        if (higherOrderFunctionType == "unweighted") {
          higherOrderSequence <- functionOverScaffoldWithWeights(
            scaffold,
            exprs[,sample(nc),drop = FALSE],
            higherOrderFunction,
            weightMatrix)
        } else {
          # for weighted
          higherOrderSequence <- weightedFunctionOverScaffold(
            scaffold,
            exprs[,sample(nc),drop = FALSE],
            higherOrderFunction,
            weightMatrix)
        }
        higherOrderSummaryFunction(higherOrderSequence)

      }, BPPARAM = BPPARAM)

      permutations <- unlist(permutations)

    } else {

      permutations <- replicate(niter, {

        if (higherOrderFunctionType == "unweighted") {
          higherOrderSequence <- functionOverScaffoldWithWeights(
            scaffold,
            exprs[,sample(nc),drop = FALSE],
            higherOrderFunction,
            weightMatrix)
        } else {
          # for weighted
          higherOrderSequence <- weightedFunctionOverScaffold(
            scaffold,
            exprs[,sample(nc),drop = FALSE],
            higherOrderFunction,
            weightMatrix)
        }
        higherOrderSummaryFunction(higherOrderSequence)
      })

    }


    if (store) {
      DF$permutations[[i]] <- permutations
    }

    pval = mean(permutations >= obs, na.rm = TRUE)

    if (pval == 0) {
      pval <- 1/(length(stats::na.omit(permutations)) + 1)
    }

    DF[i,"pvalPermutations"] <- pval

  }

  DF$FDRPermutations = stats::p.adjust(DF$pvalPermutations, method = "BH")

  # scHOT@scHOT_output <- DF
  scHOT_output(scHOT) <- DF

  if (store) {
    scHOT@scHOT_output$permutations <- IRanges::NumericList(
      scHOT@scHOT_output$permutations)
  }

  return(scHOT)

}



###################################################################
#'
#' @importFrom graphics plot
#' @importFrom reshape sort_df
#'

estimatePvalues <- function(stats,
                            globalCors,
                            permstats,
                            usenperm_estimate = FALSE,
                            nperm_estimate = 10000,
                            maxDist = 0.1,
                            # plot = FALSE,
                            verbose = FALSE) {

  # if (plot) {
  #   graphics::plot(globalCors[names(permstats)],
  #                  unlist(lapply(permstats,
  #                                function(x) mean(unlist(x)))),
  #                  xlab = "Global Spearman correlation",
  #                  ylab = "Mean of permuted statistics")
  # }


  # melt permstats list
  permstatsDF = data.frame(
    genepair = rep(names(permstats),
                   times = unlist(
                     lapply(permstats, function(x) length(unlist(x))))),
    stat = unlist(permstats))

  # add globalCor in the permstats data frame
  permstatsDF$globalCor = globalCors[as.character(permstatsDF$genepair)]


  # this can be do it in parallel
  results = sapply(names(stats), function(genepair) {

    if (verbose) {
      cat(paste0(genepair," \n"))
    }

    stat = stats[genepair]
    globalCor = globalCors[genepair]

    permstatsDF$dist = abs(permstatsDF$globalCor - globalCor)

    if (usenperm_estimate) {
      if (nperm_estimate >= nrow(permstatsDF)) {
        message("nperm_estimate given is larger than or equal to total number of permutations... using all permutations")
        permstatsDF_sorted_sub = na.omit(permstatsDF)
      }
      else {
        permstatsDF_sorted = reshape::sort_df(na.omit(permstatsDF), "dist")
        permstatsDF_sorted_sub = permstatsDF_sorted[seq_len(nperm_estimate), ]
      }
    } else {
      if (is.null(maxDist)) {
        message("usenperm_estimate set as FALSE, but maxDist not given... defaulting to maxDist = 0.1")
        maxDist = 0.1
      }

      permstatsDF_sorted_sub = subset(na.omit(permstatsDF), dist <
                                        maxDist)
      if (nrow(permstatsDF_sorted_sub) == 0) {
        message("no permutations found within given maxDist... using all permutations instead")
        permstatsDF_sorted_sub = na.omit(permstatsDF)
      }
    }

    permstats_sub = permstatsDF_sorted_sub$stat


    if (length(permstats_sub) == 0)
      stop("please set a larger maxDist, or set a larger usenperm_estimate")

    pval = mean(permstats_sub >= stat)
    if (pval == 0) {
      pval = 1/(1 + length(permstats_sub))
    }
    nperm_estimate = nrow(permstatsDF_sorted_sub)
    globalCorMin = min(permstatsDF_sorted_sub$globalCor)
    globalCorMax = max(permstatsDF_sorted_sub$globalCor)
    return(data.frame(genepair = genepair,
                      stat = stat,
                      globalCor = globalCor,
                      pval = pval,
                      nperm_estimate = nperm_estimate,
                      globalCorMin = globalCorMin,
                      globalCorMax = globalCorMax))
  }, simplify = FALSE)

  resultsDF = do.call(rbind, results)
  return(resultsDF)
}



###################################################################

#' Estimate p-values based on already run permutation tests
#'
#' @title scHOT_estimatePvalues
#'
#' @param scHOT A scHOT object
#' @param usenperm_estimate Logical (default FALSE) if
#' number of neighbouring permutations should be used, or
#' if difference of global higher order statistic should be used
#' @param nperm_estimate Number of neighbouring permutations to
#' use for p-value estimation
#' @param maxDist max difference of global higher order statistic
#' to use for p-value estimation (default 0.1)
#' @param plot A logical input indicating whether the results are plotted
#' @param verbose A logical input indicating whether the intermediate
#' steps will be printed
#'
#' @return scHOT A scHOT object with results stored in scHOT_output slot
#' @examples
#'  data(MOB_subset)
#'  sce_MOB_subset <- MOB_subset$sce_MOB_subset
#'  scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
#'                                      assayName = "logcounts",
#'                                     positionType = "spatial",
#'                                      positionColData = c("x", "y"))
#' pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
#' rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
#'
#' scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
#'
#' scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
#'                                        positionColData = c("x","y"),
#'                                         positionType = "spatial",
#'                                         nrow.out = NULL,
#'                                         span = 0.05)
#' scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
#'   scHOT_spatial,
#'   higherOrderFunction = weightedSpearman,
#'   higherOrderFunctionType = "weighted")
#' scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
#'                                               numberPermutations = 100)
#' scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(
#'   scHOT_spatial,
#'   higherOrderSummaryFunction = sd)
#'
#' scHOT_spatial <- scHOT_performPermutationTest(
#'   scHOT_spatial,
#'   verbose = TRUE,
#'   parallel = FALSE)
#'
#' scHOT_spatial <- scHOT_estimatePvalues(scHOT_spatial)
#'
#' @importFrom stats p.adjust
#'
#' @export
#'

scHOT_estimatePvalues <- function(scHOT,
                                  usenperm_estimate = FALSE,
                                  nperm_estimate = 10000,
                                  maxDist = 0.1,
                                  plot = FALSE,
                                  verbose = FALSE) {

  scHOT_output = scHOT@scHOT_output

  stats = scHOT_output$higherOrderStatistic
  globalCors = scHOT_output$globalHigherOrderFunction
  permstats = as.list(scHOT_output$permutations)

  # check the input

  if (length(stats) == 0) {
    stop("No higherOrderStatistic found in scHOT object's scHOT_output slot,
         please run function scHOT_calculateHigherOrderTestStatistics!")
  }

  if (length(globalCors) == 0) {
    stop("No globalHigherOrderFunction found in
    scHOT object's scHOT_output slot,
         please run function scHOT_calculateGlobalHigherOrderFunction!")
  }

  if (length(permstats) == 0) {
    stop("No permutations found in scHOT object's scHOT_output slot,
         please run function scHOT_performPermutationTest!")
  }

  if (plot) {
    p <- scHOT_plotPermutationDistributions(scHOT)
    plot(p)
  }

  if (is.null(names(stats))) {
    names(stats) <- paste0("stat_", seq_len(length(stats)))
  }

  names(globalCors) <- names(stats)
  names(permstats) <- names(stats)

  out = estimatePvalues(stats,
                        globalCors,
                        permstats,
                        usenperm_estimate,
                        nperm_estimate,
                        maxDist,
                        verbose)


  scHOT@scHOT_output$numberPermutationsEstimated = out$nperm
  scHOT@scHOT_output$globalLowerRangeEstimated = out$globalCorMin
  scHOT@scHOT_output$globalUpperRangeEstimated = out$globalCorMax
  scHOT@scHOT_output$pvalEstimated = out$pval
  scHOT@scHOT_output$FDREstimated = stats::p.adjust(
    scHOT@scHOT_output$pvalEstimated, method = "BH")

  return(scHOT)
}

---
title: "Getting started: scHOT"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::knitr}
  %\VignetteIndexEntry{Getting started: scHOT}
  %\usepackage[UTF-8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = FALSE,
  warning = FALSE,
  comment = "#>"
)
```

# Introduction

Single cell Higher Order Testing (scHOT) is an R package that facilitates 
testing changes in higher order structure along either a developmental
trajectory or across space. In this vignette, we go through two example 
analyses: 1) Testing variability changes along liver trajectory, and 2) Testing
differential correlation across the mouse olfactory bulb.

```{r}
library(SingleCellExperiment)
library(ggplot2)
library(scHOT)
library(scater)
library(matrixStats)
```

# Testing variability changes along liver trajectory

The `liver` dataset contains data from two branches of a developmental 
trajectory, starting from immature hepatoblasts and bifurcates into either
the hepatocyte or cholangiocyte lineages. For file size 
reasons we only load up the hepatocyte lineage for a small number of genes. 
In this example, we take the cells
that belong to the initial part of the trajectory, i.e. from hepatoblast to 
the bifurcation point, and test for differential variability along this 
trajectory for a few genes.

```{r}
data(liver)

liver_pseudotime_hep <- liver$liver_pseudotime_hep
liver_branch_hep <- liver$liver_branch_hep
first_branch_cells <- liver$first_branch_cells
```

```{r}
gene_to_test <- as.matrix(c("Birc5", "H2afz", "Tacc3"))
```

## Build the `scHOT` object

First we build the `scHOT` object, which is based on the `SingleCellExperiment` 
object class. `scHOT` objects can be built either from a matrix format or from 
an existing `SingleCellExperiment` object. In this case, we have matrix data 
so we build the `scHOT` object using `scHOT_buildFromMatrix`. Since the liver
data represents a trajectory, we set the `positionType` as `"trajectory"`, and 
provide the column name of the cell metadata (argument `cellData`) for which 
the cells should be ordered.

```{r}
scHOT_traj <- scHOT_buildFromMatrix(
  mat = liver_branch_hep[,first_branch_cells],
  cellData = list(pseudotime = liver_pseudotime_hep[first_branch_cells]),
  positionType = "trajectory",
  positionColData = "pseudotime")
scHOT_traj
```

`scHOT_traj` is a `scHOT` object, but methods associated with 
`SingleCellExperiment` can also be used. For example, we use the `scater` 
package to plot the expression of the hepatoblast marker _Sall4_ along 
pseudotime, and note that this decreases as pseudotime increases.

```{r}
scater::plotExpression(scHOT_traj, c("Sall4"),
                       exprs_values = "expression", x = "pseudotime")
```

## scHOT wrapper function

Now using the `scHOT` wrapper function, we can perform higher order testing on 
the selected genes, provided as a one-column matrix. To do this, we also need 
to set the underlying higher order function, which in this case we use weighed 
variance as implemented in the `matrixStats` package. Since this function has 
a weight parameter, we set `higherOrderFunctionType = "weighted"`. For basic 
implementation, no other parameters need to be specified (for speed, we set 
`numberPermutations` to a small value).

```{r}
scHOT_traj_wrap = scHOT(scHOT_traj,
                        testingScaffold = gene_to_test,
                        higherOrderFunction = matrixStats::weightedVar,
                        higherOrderFunctionType = "weighted",
                        numberPermutations = 50)
```

Output is saved as a `DataFrame` in the `scHOT_output` slot, accessible either 
using the `slot` function, or using the `@` accessor. In particular, we can
interrogate the higher order sequence, the sequence of locally weighted 
variances along the trajectory. We can see from the plot that each of these 
genes increases in variability along pseudotime. Note that the plots are 
based on `ggplot2` and so can be customised as desired.

```{r}
slot(scHOT_traj_wrap, "scHOT_output")
scHOT_traj_wrap@scHOT_output

slot(scHOT_traj_wrap, "scHOT_output")$higherOrderSequence

plotHigherOrderSequence(scHOT_traj_wrap, gene_to_test)

plotOrderedExpression(scHOT_traj_wrap, gene_to_test) + 
  facet_wrap(~gene, scales = "free_y")
```

## scHOT step-by-step

Now, we can perform the same testing but step-by-step, with description of the
parameter selection at each step.

First, we add the testing scaffold. This is the set of genes for which we wish
to perform higher order testing.

```{r}
scHOT_traj@testingScaffold
scHOT_traj <- scHOT_addTestingScaffold(scHOT_traj, gene_to_test)
scHOT_traj@testingScaffold
```

Next, we provide parameters to build a weighting scheme. The most important 
parameter here is `span`, a value between 0 and 1 (default 0.25) which 
determines how large or small a window we wish to use for higher order testing. 
To distinguish between either ranked samples in a trajectory or in a spatial 
setting, we set `positionType = "trajectory"` and instruct to extract the 
trajectory ordering from the `"pseudotime"` column from `colData(scHOT_traj)`.

```{r}
scHOT_traj@weightMatrix
scHOT_traj <- scHOT_setWeightMatrix(scHOT_traj,
                                    positionType = "trajectory",
                                    positionColData = c("pseudotime"),
                                    nrow.out = NULL,
                                    span = 0.25)
dim(scHOT_traj@weightMatrix)
class(scHOT_traj@weightMatrix)
plot(scHOT_traj@weightMatrix[50,])
```

By default the weight matrix is square, with as many rows as the number of 
cells (columns), but if you have especially large data, you may want to reduce 
the weight matrix rows for faster runtimes. Here we reduce the weight matrix to 
roughly 50 rows using the `nrow.out` arguemnt. Note you can provide your own 
pre-prepared matrix using the `weightMatrix` argument.

```{r}
scHOT_traj <- scHOT_setWeightMatrix(scHOT_traj,
                                    positionType = "trajectory",
                                    positionColData = c("pseudotime"),
                                    nrow.out = 50,
                                    span = 0.25)
dim(scHOT_traj@weightMatrix)
class(scHOT_traj@weightMatrix)
plot(scHOT_traj@weightMatrix[50,])
```

Now we calculate the global higher order function, in this case is simply the
sample variance giving equal weight to all cells. This becomes important when you 
wish to test many genes and use a fast p-value estimation approach to speed up 
computation.

```{r}
scHOT_traj <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_traj, 
  higherOrderFunction = matrixStats::weightedVar,
  higherOrderFunctionType = "weighted")

slot(scHOT_traj, "scHOT_output")
apply(assay(scHOT_traj, "expression")[c(gene_to_test),],1,var)
```

`scHOT` allows for a lot of customisation. In particular, you can set which
tests you wish to perform permutation testing for, and if so, what number of 
permutations can be used, and if they should be stored for later use. Here, we 
set the number of permutations to a mix of 20 and 50. Allowing different 
permutation numbers is useful if we wish to perform a lot of permutations for 
one test, and few for another.

Note if you wish to explicitly set the number of permutations for all tests, 
then ensure that `numberScaffold` is set above the number of tests (default 
100).

`scHOT_setPermutationScaffold` only gives the instructions for running 
permutation testing, it doesn't actually perform the testing.

```{r}
scHOT_traj <- scHOT_setPermutationScaffold(scHOT_traj, 
                                           numberPermutations = c(20,50,20),
                                           storePermutations = c(TRUE, FALSE, TRUE))
slot(scHOT_traj, "scHOT_output")
```

Now we can calculate the observed test statistic. This is calculated as a 
summary function (default standard deviation) of the local higher order 
sequence, which is parametrised using the higher order function (here weighted 
variance) and weighting scheme (here thinned triangular weight matrix 
with span 0.25).

```{r}
scHOT_traj <- scHOT_calculateHigherOrderTestStatistics(
  scHOT_traj,
  higherOrderSummaryFunction = sd)

slot(scHOT_traj, "scHOT_output")
```

Once the test statistic is calculated, we perform permutation testing, using
the instructions we provided earlier.

```{r}
system.time(scHOT_traj <- scHOT_performPermutationTest(
  scHOT_traj, 
  verbose = TRUE,
  parallel = FALSE))

slot(scHOT_traj, "scHOT_output")
```

To avoid P-values identically zero, we rescale zero P-values to 
1/(1+numberPermutations).

We could also use the existing stored permutations to estimate P-values, 
since genes with a similar global higher order function are likely to have a 
similar null distribution. In this example case it does not show much gain
computationally, but this can significantly reduce the number of permutation 
tests needed for large datasets and testing strategies.

Running `scHOT_estimatePvalues` results in more columns in the `scHOT_output`
slot, corresponding to the number of permutations used in estimating, the
range of the global higher order function used for estimating, as well as the 
estimated P-value itself and FDR adjusted P-value. Here, we set a maximum of 
10,000 permutations to be used, for genes with at most a difference of 5 in the 
global higher order function (the variance).

```{r}
scHOT_traj <- scHOT_estimatePvalues(scHOT_traj,
                                    nperm_estimate = 10000,
                                    maxDist = 5)
slot(scHOT_traj, "scHOT_output")
```

Note that having performed the testing using the thinned weight matrix, we can
still plot, but beware that not every position is sampled.

```{r}
plotHigherOrderSequence(scHOT_traj, gene_to_test)
```

# Spatial differential correlation in Mouse Olfactory Bulb

In this example, we look at the Spatial Transcriptomics mouse olfactory bulb 
data. This data is provided in the form of a `SingleCellExperiment` object, 
with the spatial coordinates provided in the `colData` slot. For file size 
reasons we only load up a small number of genes, corresponding to highly 
variable genes (HVGs) that are not found to be significantly 
differentially expressed in space.

```{r}
data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset

sce_MOB_subset
```

We build the `scHOT` object using `scHOT_buildFromSCE`. Note that `scHOT` only
takes in a single `assay` slot, for which testing is based on.

```{r}
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                    assayName = "logcounts",
                                    positionType = "spatial",
                                    positionColData = c("x", "y"))

scHOT_spatial
```

## Perform scHOT step by step - skip [ahead](#perform-schot-using-schot-wrapper-function) for wrapper using `scHOT`

In this example, we want to perform spatial differential correlation testing 
between distinct pairs of genes. We build up our testing scaffold by taking
all pairs of the non-differentially expressed HVGs. For practicality, we only 
consider a small set of pairs for testing here.

```{r}
pairs <- t(combn(rownames(sce_MOB_subset),2))
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
head(pairs)

set.seed(2020)
pairs <- pairs[sample(nrow(pairs), 20), ]
if (!"Arrb1_Mtor" %in% rownames(pairs)) {
pairs <- rbind(pairs, "Arrb1_Mtor" = c("Arrb1", "Mtor"))
}
if (!"Dnm1l_Fam63b" %in% rownames(pairs)) {
pairs <- rbind(pairs, "Dnm1l_Fam63b" = c("Dnm1l", "Fam63b"))
}

scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
scHOT_spatial@testingScaffold
```

Note that since we are performing differential correlation testing, our 
testing scaffold is a matrix with two columns. If you wish to use some higher
order function with more than two genes, you can simply add more columns to the 
testing scaffold, and ensure there are more arguments in the provided
higher order function.

Now we set the weight matrix, using the positional coordinates in the `scHOT` 
object. The `span` parameter here corresponds to the proportion of cells that 
have nonzero values around a radius of the central cell. 

This can also be thinned for faster computation by setting the `nrow.out` 
argument to the number of samples to roughly thin to.

```{r}
scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                       positionType = "spatial",
                                       nrow.out = NULL,
                                       span = 0.05)

dim(slot(scHOT_spatial, "weightMatrix"))
```

We can visualise the weighting scheme for each row of the weight matrix.

```{r}
cellID = 75
ggplot(as.data.frame(colData(scHOT_spatial)), aes(x = -x, y = y)) + 
  geom_point(aes(colour = slot(scHOT_spatial, "weightMatrix")[cellID,],
                 size = slot(scHOT_spatial, "weightMatrix")[cellID,])) + 
  scale_colour_gradient(low = "black", high = "purple") + 
  scale_size_continuous(range = c(1,5)) +
  theme_classic() +
  guides(colour = guide_legend(title = "Spatial Weight"),
         size = guide_legend(title = "Spatial Weight")) + 
    ggtitle(paste0("Central cell: ", cellID))
```

Now we can calculate the global higher order function. In this case, we use 
weighted Spearman correlation as our weighted higher order function, and so 
this is simply equivalent to calculating Spearman correlation for the gene pairs
of interest.

```{r}
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial, 
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")

slot(scHOT_spatial, "scHOT_output")

head(diag(cor(t(assay(scHOT_spatial, "expression")[pairs[,1],]),
         t(assay(scHOT_spatial, "expression")[pairs[,2],]),
         method = "spearman")))
```

Now we can set the permutation parameters. In this case, we only perform 50 
permutations for around 10 randomly selected tests.

```{r}
scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial, 
                                              numberPermutations = 50,
                                              numberScaffold = 10)

slot(scHOT_spatial, "scHOT_output")
```

Now we calculate the observed higher order test statistics, which in this case
correspond to the summary of local correlation vectors.

```{r}
scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(scHOT_spatial)

slot(scHOT_spatial, "scHOT_output")
```

Now we perform permutation testing for those tests which we provided with a 
nonzero value for number of permutations. This takes about a minute to run.

```{r}
system.time(scHOT_spatial <- scHOT_performPermutationTest(
  scHOT_spatial, 
  verbose = TRUE,
  parallel = FALSE))

slot(scHOT_spatial, "scHOT_output")
```

With the above, we calculated P-values for some of the tests, but we have not 
performed permutation testing for all tests. Here, we employ a
permutation sharing approach to estimate significance for the other tests. 
Ideally this would be performed with around a hundred tests each with around 
1,000 permutations to ensure accurate P-value estimation. You can check how 
good an estimate you can expect by plotting the global higher order statistic 
against the permuted test statistics, there should be good representation 
along the x-axis and the fitted curve should appear quite smooth. Here the 
fitted curve is bumpy, so we would suggest selecting more genes and more 
permutations.

We estimate P-values by borrowing 100 permutations from closest tests with a 
difference in global higher order statistic of at most 0.1

```{r}
scHOT_plotPermutationDistributions(scHOT_spatial)

scHOT_spatial <- scHOT_estimatePvalues(scHOT_spatial,
                                       nperm_estimate = 100,
                                       maxDist = 0.1)
slot(scHOT_spatial, "scHOT_output")

ggplot(as.data.frame(slot(scHOT_spatial, "scHOT_output")),
       aes(x = -log10(pvalPermutations), y = -log10(pvalEstimated))) + 
  geom_point() + 
  theme_classic() + 
  geom_abline(slope = 1, intercept = 0) +
  xlab("Permutation -log10(p-value)") +
  ylab("Estimated -log10(p-value)") +
  NULL
```

In this example we can still see fairly good concordance between the estimated 
and direct permutation tests, even with a very small number of permutations.

Once testing is done, we can interrogate the results with various plots. The 
`plotHigherOrderSequence` function will plot the points in space, coloured by
the local correlation estimates, and `plotOrderedExpression` will plot the 
points in space, coloured by expression of each gene.

```{r}
colData(scHOT_spatial)[, "-x"] <- -colData(scHOT_spatial)[, "x"]
plotHigherOrderSequence(scHOT_spatial, c("Dnm1l_Fam63b"),
                        positionColData = c("-x", "y"))

plotOrderedExpression(scHOT_spatial, c("Dnm1l", "Fam63b"),
                      positionColData = c("-x", "y"),
                      assayName = "expression")
```

## Perform scHOT using `scHOT` wrapper function

Strip the existing scHOT output using `scHOT_stripOutput` before rerunning 
scHOT in a new context.

```{r}
scHOT_spatial <- scHOT_stripOutput(scHOT_spatial, force = TRUE)

scHOT_spatial
slot(scHOT_spatial, "scHOT_output")
```

We can perform `scHOT` in a single wrapper function, which will perform the 
steps as described above, with all parameters given at once.

```{r}
scHOT_spatial <- scHOT(scHOT_spatial,
                       testingScaffold = pairs,
                       positionType = "spatial",
                       positionColData = c("x", "y"),
                       nrow.out = NULL,
                       higherOrderFunction = weightedSpearman,
                       higherOrderFunctionType = "weighted",
                       numberPermutations = 50,
                       numberScaffold = 10,
                       higherOrderSummaryFunction = sd,
                       parallel = FALSE,
                       verbose = FALSE,
                       span = 0.05)

slot(scHOT_spatial, "scHOT_output")
```

Again, we can examine the results in terms of the spatial expression and the 
local higher order sequences.

```{r}
plotOrderedExpression(scHOT_spatial, c("Arrb1", "Mtor"),
                      positionColData = c("-x", "y"),
                      assayName = "expression")

plotHigherOrderSequence(scHOT_spatial, "Arrb1_Mtor",
                        positionColData = c("-x", "y"))
```

# Misc

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MOB_subset}
\alias{MOB_subset}
\title{MOB_subset spatial example data}
\format{
An object of class \code{list} of length 1.
}
\usage{
data(MOB_subset, package = 'scHOT')
}
\description{
A MOB spatial data set.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderFunctions.R
\name{weightedZISpearman}
\alias{weightedZISpearman}
\title{weightedZISpearman}
\usage{
weightedZISpearman(x, y, w = 1)
}
\arguments{
\item{x}{x and y are non-negative data vectors}

\item{y}{x and y are non-negative data vectors}

\item{w}{weight vector, values should be between 0 and 1}
}
\value{
\code{numeric} weighted rho* association value between x and y

 Pimentel, Ronald Silva, "Kendall's Tau and Spearman's Rho for
 Zero-Inflated Data" (2009). Dissertations. 721.
 https://scholarworks.wmich.edu/dissertations/721
}
\description{
the weightedZISpearman function calculates weighted rho\*,
where rho\* is described in Pimentel et al (2009).
This association measure is defined for zero-inflated,
non-negative random variables.
}
\examples{

x = pmax(0,rnorm(100))
y = pmax(0,rnorm(100))
w = runif(100)
weightedZISpearman(x,y,w)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderFunctions.R
\name{weightedSpearman}
\alias{weightedSpearman}
\title{weightedSpearman}
\usage{
weightedSpearman(x, y, w = 1)
}
\arguments{
\item{x}{x and y are data vectors}

\item{y}{x and y are data vectors}

\item{w}{weight vector, values should be between 0 and 1}
}
\value{
\code{numeric} weighted correlation value between x and y
}
\description{
the weightedSpearman function
}
\examples{

x = rnorm(100)
y = rnorm(100)
w = runif(100)
weightedSpearman(x,y,w)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_setWeightMatrix}
\alias{scHOT_setWeightMatrix}
\title{scHOT_setWeightMatrix}
\usage{
scHOT_setWeightMatrix(
  scHOT,
  weightMatrix = NULL,
  positionType = NULL,
  positionColData = NULL,
  nrow.out = NULL,
  averageAcrossTrajectoryTies = FALSE,
  ...
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{weightMatrix}{A matrix indicating the weight matrix for
scHOT analysis, such as the output from `trajectoryWeightMatrix`
or `spatialWeightMatrix`. If this is not NULL then other parameters
are ignored.}

\item{positionType}{A string indicating the position type,
either "trajectory" or "spatial"}

\item{positionColData}{Either trajectory or spatial information
for each sample. If positionType is "trajectory"
then positionColData should be a character or
numeric indicating the subset of colData of the scHOT object.
If positionType is "spatial" then positionColData
should be a character or numeric vector indicating
the subset of colData that give the full spatial coordinates.}

\item{nrow.out}{The number of weightings to include for testing,
a smaller value is faster for computation}

\item{averageAcrossTrajectoryTies}{Logical indicating whether ties
in the trajectory should be given the same local weights}

\item{...}{parameters for function trajectoryWeightMatrix
or spatialWeightMatrix}
}
\value{
A scHOT object with slot weightMatrix saved
}
\description{
Create scHOT object from a SingleCellExperiment object
}
\examples{

 data(MOB_subset)
 sce_MOB_subset <- MOB_subset$sce_MOB_subset
 scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))
 pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
 rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
 scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
 scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                        positionType = "spatial",
                                        nrow.out = NULL,
                                        span = 0.05)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_plot.R
\name{plotEgoNetwork}
\alias{plotEgoNetwork}
\title{plotEgoNetwork}
\usage{
plotEgoNetwork(
  scHOT,
  hubnode,
  network,
  weight = "higherOrderStatistic",
  subset = FALSE,
  thresh = NULL
)
}
\arguments{
\item{scHOT}{a scHOT object}

\item{hubnode}{is a character vector of node(s) to include as hub nodes}

\item{network}{is an igraph network}

\item{weight}{A string indicates the column name stored in scHOT_output slot
that are used as the weights of the network}

\item{subset}{is a logical asking if you should subset
based on the weight (default FALSE)}

\item{thresh}{is the subset weight threshold}
}
\value{
\code{igraph} object containing the network graphed.
Produces an igraph plot
}
\description{
the plotEgoNetwork function plots network graphs with edges coloured by
weights in the network
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_stripOutput}
\alias{scHOT_stripOutput}
\title{scHOT_stripOutput}
\usage{
scHOT_stripOutput(scHOT, force = TRUE, store = FALSE, file_name = NULL)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{force}{A logical indicates whther forcing stripping the scHOT output}

\item{store}{A logical flag on whether the scHOT should be stored as .rds file}

\item{file_name}{A string indicates the file name of the scHOT will be stored}
}
\value{
A scHOT object with scHOT_output striped
}
\description{
Strip the scHOT output
}
\examples{
data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))

scHOT_spatial <- scHOT_stripOutput(scHOT_spatial)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_functions.R
\name{scHOT_calculateHigherOrderTestStatistics}
\alias{scHOT_calculateHigherOrderTestStatistics}
\title{scHOT_calculateHigherOrderTestStatistics}
\usage{
scHOT_calculateHigherOrderTestStatistics(
  scHOT,
  higherOrderSummaryFunction = stats::sd,
  ...
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{higherOrderSummaryFunction}{A function which indicates how the higher
order sequence is summarised, default is sd}

\item{...}{parameters for higherOrderSummaryFunction}
}
\value{
scHOT A scHOT object with results stored in scHOT_output slot
}
\description{
Calculate and store the higherOrderSequence and higherOrderTestStatistic
}
\examples{
data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)

rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)

scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                        positionType = "spatial",
                                        nrow.out = NULL,
                                        span = 0.05)
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial,
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")
scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
                                              numberPermutations = 100)
scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(
  scHOT_spatial,
  higherOrderSummaryFunction = sd)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT.R
\name{scHOT}
\alias{scHOT}
\title{scHOT}
\usage{
scHOT(
  scHOT,
  testingScaffold = NULL,
  weightMatrix = NULL,
  positionType = NULL,
  positionColData = NULL,
  nrow.out = NULL,
  averageAcrossTrajectoryTies = FALSE,
  higherOrderFunction = NULL,
  higherOrderFunctionType = NULL,
  numberPermutations = 1000,
  numberScaffold = 100,
  storePermutations = TRUE,
  higherOrderSummaryFunction = sd,
  parallel = FALSE,
  BPPARAM = BiocParallel::SerialParam(),
  usenperm_estimate = FALSE,
  nperm_estimate = 10000,
  maxDist = 0.1,
  plot = FALSE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{testingScaffold}{A matrix with rows for each testing combination}

\item{weightMatrix}{A matrix indicates the weight matrix for scHOT analysis}

\item{positionType}{A string indicating the position type,
either "trajectory" or "spatial"}

\item{positionColData}{Either trajectory or spatial information
for each sample. If positionType is "trajectory"
then positionColData should be a character or numeric indicating
the subset of colData of the scHOT object.
If positionType is "spatial" then positionColData should be
a character or numeric vector indicating the subset of colData that
give the full spatial coordinates.}

\item{nrow.out}{The number of weightings to include for testing,
a smaller value is faster for computation}

\item{averageAcrossTrajectoryTies}{Logical indicating whether ties
in the trajectory should be given the same local weights}

\item{higherOrderFunction}{A function object indicates the
higher order function}

\item{higherOrderFunctionType}{is "weighted" or "unweighted",
determines if there
is a weighting argument in the higher order function}

\item{numberPermutations}{The number of permutations,
set as 1000 by default}

\item{numberScaffold}{The number of testing scaffolds to
perform permutations, set as 100 by default}

\item{storePermutations}{a logical flag on whether
permutation values should be saved}

\item{higherOrderSummaryFunction}{A functon indicating the higher order
summary function (default is standard deviation `sd`)}

\item{parallel}{A logical input indicating whether to run
the permutation test using multiple cores in parallel.}

\item{BPPARAM}{A \code{BiocParallelParam} class object from
the \code{BiocParallel} package is used. Default is SerialParam().}

\item{usenperm_estimate}{Logical (default FALSE) if number of neighbouring
permutations should be used to estimate P-values, or
if difference of global higher order statistic should be used}

\item{nperm_estimate}{Number of neighbouring permutations to
use for p-value estimation}

\item{maxDist}{max difference of global higher order statistic to
use for p-value estimation (default 0.1)}

\item{plot}{A logical input indicating whether the results are plotted}

\item{verbose}{A logical input indicating whether the intermediate
steps will be printed}

\item{...}{parameters for function trajectoryWeightMatrix or spatialWeightMatrix}
}
\value{
A scHOT object
}
\description{
A wrapper function to perform scHOT
}
\examples{
data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                    assayName = "logcounts",
                                    positionType = "spatial",
                                    positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")

scHOT_spatial <- scHOT(scHOT_spatial,
                       testingScaffold = pairs,
                       positionType = "spatial",
                       positionColData = c("x", "y"),
                       nrow.out = NULL,
                       higherOrderFunction = weightedSpearman,
                       higherOrderFunctionType = "weighted",
                       numberPermutations = 100,
                       higherOrderSummaryFunction = sd,
                       parallel = FALSE,
                       verbose = TRUE,
                       span = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_functions.R
\name{scHOT_performPermutationTest}
\alias{scHOT_performPermutationTest}
\title{scHOT_performPermutationTest}
\usage{
scHOT_performPermutationTest(
  scHOT,
  verbose = FALSE,
  parallel = FALSE,
  BPPARAM = BiocParallel::SerialParam()
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{verbose}{A logical input indicates whether the intermediate steps
will be printed}

\item{parallel}{A logical input indicates whether run the permutation test
using multiple cores in parallel.}

\item{BPPARAM}{A \code{BiocParallelParam} class object
from the \code{BiocParallel} package is used. Default is SerialParam().}
}
\value{
scHOT A scHOT object with results stored in scHOT_output slot
}
\description{
Perform permutation test
}
\examples{
 data(MOB_subset)
 sce_MOB_subset <- MOB_subset$sce_MOB_subset
 scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
 scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                        positionType = "spatial",
                                        nrow.out = NULL,
                                        span = 0.05)
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial,
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")
scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
                                              numberPermutations = 100)
scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(
  scHOT_spatial,
  higherOrderSummaryFunction = sd)

scHOT_spatial <- scHOT_performPermutationTest(
  scHOT_spatial,
  verbose = TRUE,
  parallel = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_functions.R
\name{scHOT_estimatePvalues}
\alias{scHOT_estimatePvalues}
\title{scHOT_estimatePvalues}
\usage{
scHOT_estimatePvalues(
  scHOT,
  usenperm_estimate = FALSE,
  nperm_estimate = 10000,
  maxDist = 0.1,
  plot = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{usenperm_estimate}{Logical (default FALSE) if
number of neighbouring permutations should be used, or
if difference of global higher order statistic should be used}

\item{nperm_estimate}{Number of neighbouring permutations to
use for p-value estimation}

\item{maxDist}{max difference of global higher order statistic
to use for p-value estimation (default 0.1)}

\item{plot}{A logical input indicating whether the results are plotted}

\item{verbose}{A logical input indicating whether the intermediate
steps will be printed}
}
\value{
scHOT A scHOT object with results stored in scHOT_output slot
}
\description{
Estimate p-values based on already run permutation tests
}
\examples{
 data(MOB_subset)
 sce_MOB_subset <- MOB_subset$sce_MOB_subset
 scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")

scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)

scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                        positionType = "spatial",
                                        nrow.out = NULL,
                                        span = 0.05)
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial,
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")
scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
                                              numberPermutations = 100)
scHOT_spatial <- scHOT_calculateHigherOrderTestStatistics(
  scHOT_spatial,
  higherOrderSummaryFunction = sd)

scHOT_spatial <- scHOT_performPermutationTest(
  scHOT_spatial,
  verbose = TRUE,
  parallel = FALSE)

scHOT_spatial <- scHOT_estimatePvalues(scHOT_spatial)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_class.R
\docType{class}
\name{scHOT-class}
\alias{scHOT-class}
\alias{.scHOT}
\title{scHOT class}
\description{
scHOT class
}
\section{Slots}{

\describe{
\item{\code{testingScaffold}}{A matrix with rows for each testing combination}

\item{\code{weightMatrix}}{A matrix or dgCMatrix indicates the weight matrix}

\item{\code{scHOT_output}}{A data.frame or DtatFrame to store output from scHOT}

\item{\code{params}}{A list of parameters}

\item{\code{positionType}}{A character indicates the type of the position,
either trajectory or spatial}

\item{\code{positionColData}}{A vector indicates column names of colData
that stored the postion informaton}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{liver}
\alias{liver}
\title{Liver trajectory example data}
\format{
A `list` object.
}
\usage{
data(liver, package = 'scHOT')
}
\description{
A liver data set contains two branches
(hepatocyte and cholangiocyte).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_setPermutationScaffold}
\alias{scHOT_setPermutationScaffold}
\title{scHOT_setPermutationScaffold}
\usage{
scHOT_setPermutationScaffold(
  scHOT,
  numberPermutations = 1000,
  numberScaffold = 100,
  storePermutations = TRUE
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{numberPermutations}{The number of permutations, set as 1000 by default}

\item{numberScaffold}{The number of scaffold, set as 100 by default, minimum 6.
if you want all combinations to do permutations then set,
numberScaffold much higher than the testingScaffold}

\item{storePermutations}{a logical flag on whether
Permutations should be stored, or discarded once used}
}
\value{
A scHOT object with storePermutations in slot scHOT_output saved
}
\description{
Set permutation scaffold
}
\examples{
 data(MOB_subset)
 sce_MOB_subset <- MOB_subset$sce_MOB_subset
 scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                        positionType = "spatial",
                                        nrow.out = NULL,
                                        span = 0.05)
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial,
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")
scHOT_spatial <- scHOT_setPermutationScaffold(scHOT_spatial,
                                              numberPermutations = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{spatialWeightMatrix}
\alias{spatialWeightMatrix}
\title{spatialWeightMatrix}
\usage{
spatialWeightMatrix(x, span = NULL)
}
\arguments{
\item{x}{a matrix with rows corresponding to cells and columns
corresponding to dimensions to calculate Euclidean distance}

\item{span}{proportion of samples to include on either side,
default is 13/(number of rows in `x`), corresponding roughly
to points within a diamond shape distance away}
}
\value{
A weighted matrix
}
\description{
Create weight matrix for spatial data
}
\examples{

spat_x <- rnorm(50)
spat_y <- rnorm(50)
spat_coord <- cbind(spat_x, spat_y)
W <- spatialWeightMatrix(spat_coord)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_plot.R
\name{scHOT_plotPermutationDistributions}
\alias{scHOT_plotPermutationDistributions}
\title{scHOT_plotPermutationDistributions}
\usage{
scHOT_plotPermutationDistributions(scHOT)
}
\arguments{
\item{scHOT}{a scHOT object}
}
\value{
\code{ggplot} graph of global higher order function and the
permutation scHOT test statistics. This should have a continuous pattern
to be reliably used for p-value estimation
}
\description{
the scHOT_plotPermutationDistributions function plots
the permutation test statistics as a diagnostic plot
for estimating p-values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_plot.R
\name{plotOrderedExpression}
\alias{plotOrderedExpression}
\title{plotOrderedExpression}
\usage{
plotOrderedExpression(
  scHOT,
  genes,
  positionType = NULL,
  branches = NULL,
  ranked_by = NULL,
  xvals = NULL,
  subsetBranch = NULL,
  facet = FALSE,
  positionColData = NULL,
  assayName = NULL
)
}
\arguments{
\item{scHOT}{A scHOT object, where the expression
data is stored in the assay slot, with assay name "expression".}

\item{genes}{is a character vector for gene names}

\item{positionType}{A string indicates the position type,
either trajectory or spatial}

\item{branches}{A character indicates that the colnames stored
the branch information in colData}

\item{ranked_by}{A character indicates that the colnames stored
the ranking information of the cells in colData, such as trajectory time}

\item{xvals}{A character indicates that the colnames stored in
colData of the x-values associated with the samples in branchData}

\item{subsetBranch}{subsetBranch is a character vector containing
the names of the branches to be plotted. If NULL it will plot all branches}

\item{facet}{can either be FALSE, "branch", "gene", or "both"}

\item{positionColData}{A vector indicates column names of colData that
stored the postion informaton (for spatial type of data)}

\item{assayName}{the name of the assay that are used to plot.}
}
\value{
\code{ggplot} a ggplot object for ribbon plot with points
}
\description{
the plotOrderedExpression function plots expression vectors
along branches and genes as ribbon plots
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_calculateGlobalHigherOrderFunction}
\alias{scHOT_calculateGlobalHigherOrderFunction}
\title{scHOT_calculateGlobalHigherOrderFunction}
\usage{
scHOT_calculateGlobalHigherOrderFunction(
  scHOT,
  higherOrderFunction = NULL,
  higherOrderFunctionType = NULL
)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{higherOrderFunction}{A function object indicating the higher order function}

\item{higherOrderFunctionType}{is "weighted" or "unweighted", determines if there
is a weighting argument in the higher order function}
}
\value{
A scHOT object with scHOT_output$globalHigherOrderFunction in
slot scHOT_output saved
}
\description{
this calculates the global higher order function
and stores it in the output
if these aren't found in the params slot
then they need to be specified here
}
\details{
Calculates the global higher order function
}
\examples{

 data(MOB_subset)
 sce_MOB_subset <- MOB_subset$sce_MOB_subset
 scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
                                     assayName = "logcounts",
                                    positionType = "spatial",
                                     positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
rownames(pairs) <- apply(pairs,1,paste0,collapse = "_")
scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)
 scHOT_spatial <- scHOT_setWeightMatrix(scHOT_spatial,
                                       positionColData = c("x","y"),
                                        positionType = "spatial",
                                        nrow.out = NULL,
                                        span = 0.05)
scHOT_spatial <- scHOT_calculateGlobalHigherOrderFunction(
  scHOT_spatial,
  higherOrderFunction = weightedSpearman,
  higherOrderFunctionType = "weighted")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{thin}
\alias{thin}
\title{thin}
\usage{
thin(W, n = 100)
}
\arguments{
\item{W}{matrix}

\item{n}{rough number of rows to keep}
}
\value{
\code{matrix} of thinned matrix keeping only roughly n rows.
}
\description{
The thin function extracts the rows of a matrix evenly so that
roughly n number of rows remain. Used for thinning down
the weight matrix to speed up overall computation.
}
\examples{

W = trajectoryWeightMatrix(500)
W_small = thin(W, n = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_addTestingScaffold}
\alias{scHOT_addTestingScaffold}
\title{scHOT_addTestingScaffold}
\usage{
scHOT_addTestingScaffold(scHOT, testingScaffold)
}
\arguments{
\item{scHOT}{A scHOT object}

\item{testingScaffold}{A matrix with rows for each testing combination,
and columns for level of dimensionality (1 for single gene etc.)}
}
\value{
A scHOT object with slot testingScaffold saved
}
\description{
Add a testing scaffold to a scHOT object
}
\examples{

data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
assayName = "logcounts",
positionType = "spatial",
positionColData = c("x", "y"))
pairs <- matrix(c("Arrb1", "Mtor", "Dnm1l", "Gucy1b3"), ncol = 2, byrow = TRUE)
scHOT_spatial <- scHOT_addTestingScaffold(scHOT_spatial, pairs)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_buildFromMatrix}
\alias{scHOT_buildFromMatrix}
\title{scHOT_buildFromMatrix}
\usage{
scHOT_buildFromMatrix(
  mat,
  cellData = NULL,
  positionType = NULL,
  positionColData = NULL
)
}
\arguments{
\item{mat}{A matrix with rows for genes and columns for cells}

\item{cellData}{A dataframe or DataFrame object with rows for cells}

\item{positionType}{A string indicating the position type,
either "trajectory" or "spatial"}

\item{positionColData}{Strings indicate the position information
stored in colData. If positionType is "trajectory" then
positionColData should be a sortable vector
if positionType is "spatial" then positionColData should
be a matrix type object.}
}
\value{
A scHOT object
}
\description{
Create scHOT object from a matrix
}
\examples{

dat <- rbind(rnorm(50), rnorm(50), rnorm(50))
colnames(dat) <- paste0("cell_", 1:ncol(dat))
rownames(dat) <- c("gene_1","gene_2", "gene_2")

scHOT <- scHOT_buildFromMatrix(dat, cellData = data.frame(1:ncol(dat)))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{positionColData<-}
\alias{positionColData<-}
\title{Setter functions for scHOT objects}
\usage{
positionColData(x) <- value
}
\arguments{
\item{x}{A scHOT object}

\item{value}{The value the slot should take}
}
\value{
A scHOT object
}
\description{
Setter functions for scHOT objects
}
\examples{

data(MOB_subset)
sce_MOB_subset <- MOB_subset$sce_MOB_subset
scHOT_spatial <- scHOT_buildFromSCE(sce_MOB_subset,
assayName = "logcounts",
positionType = "spatial",
positionColData = c("x", "y"))

positionColData(scHOT_spatial) <- c("x", "y")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{scHOT_buildFromSCE}
\alias{scHOT_buildFromSCE}
\title{scHOT_buildFromSCE}
\usage{
scHOT_buildFromSCE(
  sce,
  assayName = "counts",
  positionType = NULL,
  positionColData = NULL
)
}
\arguments{
\item{sce}{A SingleCellExperiment object}

\item{assayName}{is a single assay to pull out from sce as
the expression matrix input of scHOT}

\item{positionType}{A string indicates the position type,
either trajectory or spatial}

\item{positionColData}{Strings indicate the position information
stored in colData.
If positionType is "trajectory" then positionColData
should be a sortable vector
if positionType is "spatial" then positionColData
should be a matrix type object.}
}
\value{
A scHOT object
}
\description{
Create scHOT object from a SingleCellExperiment object
}
\examples{

library(SingleCellExperiment)
dat <- rbind(rnorm(50), rnorm(50), rnorm(50))
colnames(dat) <- paste0("cell_", 1:ncol(dat))
rownames(dat) <- c("gene_1","gene_2", "gene_2")

sce <- SingleCellExperiment::SingleCellExperiment(assays =
S4Vectors::SimpleList(counts = dat))
scHOT <- scHOT_buildFromSCE(sce)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{trajectoryWeightMatrix}
\alias{trajectoryWeightMatrix}
\title{trajectoryWeightMatrix}
\usage{
trajectoryWeightMatrix(n, type = NULL, span = NULL)
}
\arguments{
\item{n}{indicates the number of cels}

\item{type}{Type of weight matrix, one of "triangular" (default),
"block", and "harmonic"}

\item{span}{proportion of samples to include on either side, default is 0.25}
}
\value{
A weighted matrix
}
\description{
Create weight matrix for trajectory data
}
\examples{

W <- trajectoryWeightMatrix(100)
W <- trajectoryWeightMatrix(100, type = "triangular")
W <- trajectoryWeightMatrix(100, type = "block")
W <- trajectoryWeightMatrix(100, type = "harmonic")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderFunctions.R
\name{weightedZIKendall}
\alias{weightedZIKendall}
\title{weightedZIKendall}
\usage{
weightedZIKendall(x, y, w = 1)
}
\arguments{
\item{x}{x and y are non-negative data vectors}

\item{y}{x and y are non-negative data vectors}

\item{w}{weight vector, values should be between 0 and 1}
}
\value{
\code{numeric} weighted Tau* association value between x and y
}
\description{
the weightedZIKendall function calculates weighted Tau*,
where Tau* is described in Pimentel et al (2015)
doi:10.1016/j.spl.2014.09.002. This association measure
is defined for zero-inflated, non-negative random variables.
}
\examples{

x = pmax(0,rnorm(100))
y = pmax(0,rnorm(100))
w = runif(100)
weightedZIKendall(x,y,w)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_plot.R
\name{plotColouredExpression}
\alias{plotColouredExpression}
\title{plotColouredExpression}
\usage{
plotColouredExpression(
  scHOT,
  genes,
  genes_delimeter = "_",
  branches = NULL,
  ranked_by = NULL,
  subsetBranch = NULL,
  n = 3,
  fittedline = TRUE,
  assayName = NULL
)
}
\arguments{
\item{scHOT}{A scHOT object.}

\item{genes}{is either a single character string with a delimeter,
or a length two character vector}

\item{genes_delimeter}{is the delimeter to split into two gene names
if genes is provided as a single character}

\item{branches}{A character indicates that the colnames stored
the branch information in colData}

\item{ranked_by}{A character indicates that the colnames stored
the ranking information of the cells in colData, such as trajectory time,
if it is NULL, it will be ranked based on the branch information.}

\item{subsetBranch}{subsetBranch is a character vector containing
the names of the branches to be plotted. If NULL it will plot all branches}

\item{n}{number of panels to split ranked samples into, default 3.}

\item{fittedline}{logical default TRUE, add a lm straight line to the plot}

\item{assayName}{the name of the assay that are used to plot.}
}
\value{
\code{ggplot} a ggplot object of scatterplots of expression
split by sample ordering
}
\description{
the plotColouredExpression function plots an n-panel scatterplot
of the gene pairs split by early, mid, and late in the sample ordering.
}
\examples{

data(liver)

scHOT_traj <- scHOT_buildFromMatrix(
mat = liver$liver_branch_hep,
cellData = list(pseudotime = liver$liver_pseudotime_hep),
positionType = "trajectory",
positionColData = "pseudotime")

scHOT_traj

plotColouredExpression(scHOT_traj, c("Cdt1","Top2a"), n = 5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higherOrderFunctions.R
\name{weightedVariance}
\alias{weightedVariance}
\title{weightedVariance}
\usage{
weightedVariance(x, y = NULL, w)
}
\arguments{
\item{x}{x is a data vector}

\item{y}{default to NULL, if given it is ignored}

\item{w}{weight vector, values should be between 0 and 1}
}
\value{
\code{numeric} weighted variance value for x
}
\description{
the weightedVariance function
}
\examples{

x = rnorm(100)
w = runif(100)
weightedVariance(x,w = w)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scHOT_plot.R
\name{plotHigherOrderSequence}
\alias{plotHigherOrderSequence}
\title{plotHigherOrderSequence}
\usage{
plotHigherOrderSequence(
  scHOT,
  gene,
  positionType = NULL,
  branches = NULL,
  positionColData = NULL
)
}
\arguments{
\item{scHOT}{A scHOT object with higherOrderSequence in scHOT_output slot}

\item{gene}{is either a logical vector matching rows of entries
in wcorsList, or a character of a gene}

\item{positionType}{A string indicates the position type,
either trajectory or spatial}

\item{branches}{A character indicates that the colnames stored
the branch information in colData (for trajectory type of data)}

\item{positionColData}{A vector indicates column names of colData
that stored the postion informaton (for spatial type of data)}
}
\value{
\code{ggplot} object with line plots
}
\description{
the plotHigherOrderSequence function plots weighted higher order
statistic vectors (stored in higherOrderSequence) as line plots
}
\examples{
data(liver)

scHOT_traj <- scHOT_buildFromMatrix(
mat = liver$liver_branch_hep,
cellData = list(pseudotime = liver$liver_pseudotime_hep),
positionType = "trajectory",
positionColData = "pseudotime")
scHOT_traj
plotColouredExpression(scHOT_traj, c("Cdt1","Top2a"), n = 5)

scHOT_traj <- scHOT_addTestingScaffold(scHOT_traj,
t(as.matrix(c("Cdt1", "Top2a"))))
scHOT_traj <- scHOT_setWeightMatrix(scHOT_traj,
positionColData = c("pseudotime"),
positionType = "trajectory",
nrow.out = NULL,
span = 0.25)
scHOT_traj <- scHOT_calculateGlobalHigherOrderFunction(scHOT_traj,
                                                      higherOrderFunction =
                                                      weightedSpearman,
                                                     higherOrderFunctionType =
                                                     "weighted")

scHOT_traj <- scHOT_calculateHigherOrderTestStatistics(scHOT_traj,
                                                      higherOrderSummaryFunction =
                                                      sd)

slot(scHOT_traj, "scHOT_output")
plotHigherOrderSequence(scHOT_traj, c("Cdt1_Top2a"))

}
